@@ -24,28 +24,18 @@ from __future__ import annotations
 __all__ = ["DatasetType", "SerializedDatasetType"]
 
 import re
+from collections.abc import Callable, Iterable, Mapping
 from copy import deepcopy
 from types import MappingProxyType
-from typing import (
-    TYPE_CHECKING,
-    Any,
-    Callable,
-    ClassVar,
-    Dict,
-    Iterable,
-    List,
-    Mapping,
-    Optional,
-    Tuple,
-    Type,
-    Union,
-)
+from typing import TYPE_CHECKING, Any, ClassVar
 
-from pydantic import BaseModel, StrictBool, StrictStr
+from lsst.daf.butler._compat import _BaseModelCompat
+from pydantic import StrictBool, StrictStr
 
 from ..configSupport import LookupKey
 from ..dimensions import DimensionGraph, SerializedDimensionGraph
 from ..json import from_json_pydantic, to_json_pydantic
+from ..persistenceContext import PersistenceContextVars
 from ..storageClass import StorageClass, StorageClassFactory
 
 if TYPE_CHECKING:
@@ -53,19 +43,19 @@ if TYPE_CHECKING:
     from ..dimensions import Dimension, DimensionUniverse
 
 
-def _safeMakeMappingProxyType(data: Optional[Mapping]) -> Mapping:
+def _safeMakeMappingProxyType(data: Mapping | None) -> Mapping:
     if data is None:
         data = {}
     return MappingProxyType(data)
 
 
-class SerializedDatasetType(BaseModel):
+class SerializedDatasetType(_BaseModelCompat):
     """Simplified model of a `DatasetType` suitable for serialization."""
 
     name: StrictStr
-    storageClass: Optional[StrictStr] = None
-    dimensions: Optional[SerializedDimensionGraph] = None
-    parentStorageClass: Optional[StrictStr] = None
+    storageClass: StrictStr | None = None
+    dimensions: SerializedDimensionGraph | None = None
+    parentStorageClass: StrictStr | None = None
     isCalibration: StrictBool = False
 
     @classmethod
@@ -73,9 +63,9 @@ class SerializedDatasetType(BaseModel):
         cls,
         *,
         name: str,
-        storageClass: Optional[str] = None,
-        dimensions: Optional[Dict] = None,
-        parentStorageClass: Optional[str] = None,
+        storageClass: str | None = None,
+        dimensions: dict | None = None,
+        parentStorageClass: str | None = None,
         isCalibration: bool = False,
     ) -> SerializedDatasetType:
         """Construct a `SerializedDatasetType` directly without validators.
@@ -86,22 +76,25 @@ class SerializedDatasetType(BaseModel):
 
         This method should only be called when the inputs are trusted.
         """
-        node = SerializedDatasetType.__new__(cls)
-        setter = object.__setattr__
-        setter(node, "name", name)
-        setter(node, "storageClass", storageClass)
-        setter(
-            node,
-            "dimensions",
-            dimensions if dimensions is None else SerializedDimensionGraph.direct(**dimensions),
+        cache = PersistenceContextVars.serializedDatasetTypeMapping.get()
+        key = (name, storageClass or "")
+        if cache is not None and (type_ := cache.get(key, None)) is not None:
+            return type_
+
+        serialized_dimensions = (
+            SerializedDimensionGraph.direct(**dimensions) if dimensions is not None else None
         )
-        setter(node, "parentStorageClass", parentStorageClass)
-        setter(node, "isCalibration", isCalibration)
-        setter(
-            node,
-            "__fields_set__",
-            {"name", "storageClass", "dimensions", "parentStorageClass", "isCalibration"},
+
+        node = cls.model_construct(
+            name=name,
+            storageClass=storageClass,
+            dimensions=serialized_dimensions,
+            parentStorageClass=parentStorageClass,
+            isCalibration=isCalibration,
         )
+
+        if cache is not None:
+            cache[key] = node
         return node
 
 
@@ -180,16 +173,16 @@ class DatasetType:
         compTypeName : `str`
             Name to use for component DatasetType.
         """
-        return "{}.{}".format(datasetTypeName, componentName)
+        return f"{datasetTypeName}.{componentName}"
 
     def __init__(
         self,
         name: str,
-        dimensions: Union[DimensionGraph, Iterable[Union[Dimension, str]]],
-        storageClass: Union[StorageClass, str],
-        parentStorageClass: Optional[Union[StorageClass, str]] = None,
+        dimensions: DimensionGraph | Iterable[Dimension | str],
+        storageClass: StorageClass | str,
+        parentStorageClass: StorageClass | str | None = None,
         *,
-        universe: Optional[DimensionUniverse] = None,
+        universe: DimensionUniverse | None = None,
         isCalibration: bool = False,
     ):
         if self.VALID_NAME_REGEX.match(name) is None:
@@ -204,9 +197,9 @@ class DatasetType:
         self._dimensions = dimensions
         if name in self._dimensions.universe.getGovernorDimensions().names:
             raise ValueError(f"Governor dimension name {name} cannot be used as a dataset type name.")
-        if not isinstance(storageClass, (StorageClass, str)):
+        if not isinstance(storageClass, StorageClass | str):
             raise ValueError(f"StorageClass argument must be StorageClass or str. Got {storageClass}")
-        self._storageClass: Optional[StorageClass]
+        self._storageClass: StorageClass | None
         if isinstance(storageClass, StorageClass):
             self._storageClass = storageClass
             self._storageClassName = storageClass.name
@@ -214,10 +207,10 @@ class DatasetType:
             self._storageClass = None
             self._storageClassName = storageClass
 
-        self._parentStorageClass: Optional[StorageClass] = None
-        self._parentStorageClassName: Optional[str] = None
+        self._parentStorageClass: StorageClass | None = None
+        self._parentStorageClassName: str | None = None
         if parentStorageClass is not None:
-            if not isinstance(storageClass, (StorageClass, str)):
+            if not isinstance(storageClass, StorageClass | str):
                 raise ValueError(
                     f"Parent StorageClass argument must be StorageClass or str. Got {parentStorageClass}"
                 )
@@ -380,7 +373,7 @@ class DatasetType:
         return self._storageClassName
 
     @property
-    def parentStorageClass(self) -> Optional[StorageClass]:
+    def parentStorageClass(self) -> StorageClass | None:
         """Return the storage class of the composite containing this component.
 
         Note that if DatasetType was constructed with a name of a
@@ -406,7 +399,7 @@ class DatasetType:
         return self._isCalibration
 
     @staticmethod
-    def splitDatasetTypeName(datasetTypeName: str) -> Tuple[str, Optional[str]]:
+    def splitDatasetTypeName(datasetTypeName: str) -> tuple[str, str | None]:
         """Return the root name and the component from a composite name.
 
         Parameters
@@ -434,7 +427,7 @@ class DatasetType:
             root, comp = root.split(".", maxsplit=1)
         return root, comp
 
-    def nameAndComponent(self) -> Tuple[str, Optional[str]]:
+    def nameAndComponent(self) -> tuple[str, str | None]:
         """Return the root name of this dataset type and any component.
 
         Returns
@@ -446,7 +439,7 @@ class DatasetType:
         """
         return self.splitDatasetTypeName(self.name)
 
-    def component(self) -> Optional[str]:
+    def component(self) -> str | None:
         """Return the component name (if defined).
 
         Returns
@@ -531,7 +524,7 @@ class DatasetType:
             isCalibration=self.isCalibration(),
         )
 
-    def makeAllComponentDatasetTypes(self) -> List[DatasetType]:
+    def makeAllComponentDatasetTypes(self) -> list[DatasetType]:
         """Return all component dataset types for this composite.
 
         Returns
@@ -607,7 +600,7 @@ class DatasetType:
         """
         return self.storageClass.isComposite()
 
-    def _lookupNames(self) -> Tuple[LookupKey, ...]:
+    def _lookupNames(self) -> tuple[LookupKey, ...]:
         """Return name keys to use for lookups in configurations.
 
         The names are returned in order of priority.
@@ -622,7 +615,7 @@ class DatasetType:
             composite.
         """
         rootName, componentName = self.nameAndComponent()
-        lookups: Tuple[LookupKey, ...] = (LookupKey(name=self.name),)
+        lookups: tuple[LookupKey, ...] = (LookupKey(name=self.name),)
         if componentName is not None:
             lookups = lookups + (LookupKey(name=rootName),)
 
@@ -652,7 +645,7 @@ class DatasetType:
         simple : `SerializedDatasetType`
             The object converted to a class suitable for serialization.
         """
-        as_dict: Dict[str, Any]
+        as_dict: dict[str, Any]
         if minimal:
             # Only needs the name.
             as_dict = {"name": self.name}
@@ -673,8 +666,8 @@ class DatasetType:
     def from_simple(
         cls,
         simple: SerializedDatasetType,
-        universe: Optional[DimensionUniverse] = None,
-        registry: Optional[Registry] = None,
+        universe: DimensionUniverse | None = None,
+        registry: Registry | None = None,
     ) -> DatasetType:
         """Construct a new object from the simplified form.
 
@@ -697,6 +690,13 @@ class DatasetType:
         datasetType : `DatasetType`
             Newly-constructed object.
         """
+        # check to see if there is a cache, and if there is, if there is a
+        # cached dataset type
+        cache = PersistenceContextVars.loadedTypes.get()
+        key = (simple.name, simple.storageClass or "")
+        if cache is not None and (type_ := cache.get(key, None)) is not None:
+            return type_
+
         if simple.storageClass is None:
             # Treat this as minimalist representation
             if registry is None:
@@ -720,7 +720,7 @@ class DatasetType:
             # mypy hint
             raise ValueError(f"Dimensions must be specified in {simple}")
 
-        return cls(
+        newType = cls(
             name=simple.name,
             dimensions=DimensionGraph.from_simple(simple.dimensions, universe=universe),
             storageClass=simple.storageClass,
@@ -728,14 +728,17 @@ class DatasetType:
             parentStorageClass=simple.parentStorageClass,
             universe=universe,
         )
+        if cache is not None:
+            cache[key] = newType
+        return newType
 
     to_json = to_json_pydantic
     from_json: ClassVar = classmethod(from_json_pydantic)
 
     def __reduce__(
         self,
-    ) -> Tuple[
-        Callable, Tuple[Type[DatasetType], Tuple[str, DimensionGraph, str, Optional[str]], Dict[str, bool]]
+    ) -> tuple[
+        Callable, tuple[type[DatasetType], tuple[str, DimensionGraph, str, str | None], dict[str, bool]]
     ]:
         """Support pickling.
 
