@@ -30,26 +30,16 @@ __all__ = ("DataCoordinate", "DataId", "DataIdKey", "DataIdValue", "SerializedDa
 
 import numbers
 from abc import abstractmethod
-from typing import (
-    TYPE_CHECKING,
-    AbstractSet,
-    Any,
-    ClassVar,
-    Dict,
-    Iterator,
-    Literal,
-    Mapping,
-    Optional,
-    Tuple,
-    Union,
-    overload,
-)
+from collections.abc import Iterator, Mapping, Set
+from typing import TYPE_CHECKING, Any, ClassVar, Literal, overload
 
+from deprecated.sphinx import deprecated
+from lsst.daf.butler._compat import _BaseModelCompat
 from lsst.sphgeom import IntersectionRegion, Region
-from pydantic import BaseModel
 
 from ..json import from_json_pydantic, to_json_pydantic
 from ..named import NamedKeyDict, NamedKeyMapping, NamedValueAbstractSet, NameLookupMapping
+from ..persistenceContext import PersistenceContextVars
 from ..timespan import Timespan
 from ._elements import Dimension, DimensionElement
 from ._graph import DimensionGraph
@@ -59,26 +49,28 @@ if TYPE_CHECKING:  # Imports needed only for type annotations; may be circular.
     from ...registry import Registry
     from ._universe import DimensionUniverse
 
-DataIdKey = Union[str, Dimension]
+DataIdKey = str | Dimension
 """Type annotation alias for the keys that can be used to index a
 DataCoordinate.
 """
 
 # Pydantic will cast int to str if str is first in the Union.
-DataIdValue = Union[int, str, None]
+DataIdValue = int | str | None
 """Type annotation alias for the values that can be present in a
 DataCoordinate or other data ID.
 """
 
 
-class SerializedDataCoordinate(BaseModel):
+class SerializedDataCoordinate(_BaseModelCompat):
     """Simplified model for serializing a `DataCoordinate`."""
 
-    dataId: Dict[str, DataIdValue]
-    records: Optional[Dict[str, SerializedDimensionRecord]] = None
+    dataId: dict[str, DataIdValue]
+    records: dict[str, SerializedDimensionRecord] | None = None
 
     @classmethod
-    def direct(cls, *, dataId: Dict[str, DataIdValue], records: Dict[str, Dict]) -> SerializedDataCoordinate:
+    def direct(
+        cls, *, dataId: dict[str, DataIdValue], records: dict[str, dict] | None
+    ) -> SerializedDataCoordinate:
         """Construct a `SerializedDataCoordinate` directly without validators.
 
         This differs from the pydantic "construct" method in that the arguments
@@ -87,21 +79,24 @@ class SerializedDataCoordinate(BaseModel):
 
         This method should only be called when the inputs are trusted.
         """
-        node = SerializedDataCoordinate.__new__(cls)
-        setter = object.__setattr__
-        setter(node, "dataId", dataId)
-        setter(
-            node,
-            "records",
-            records
-            if records is None
-            else {k: SerializedDimensionRecord.direct(**v) for k, v in records.items()},
-        )
-        setter(node, "__fields_set__", {"dataId", "records"})
+        key = (frozenset(dataId.items()), records is not None)
+        cache = PersistenceContextVars.serializedDataCoordinateMapping.get()
+        if cache is not None and (result := cache.get(key)) is not None:
+            return result
+
+        if records is None:
+            serialized_records = None
+        else:
+            serialized_records = {k: SerializedDimensionRecord.direct(**v) for k, v in records.items()}
+
+        node = cls.model_construct(dataId=dataId, records=serialized_records)
+
+        if cache is not None:
+            cache[key] = node
         return node
 
 
-def _intersectRegions(*args: Region) -> Optional[Region]:
+def _intersectRegions(*args: Region) -> Region | None:
     """Return the intersection of several regions.
 
     For internal use by `ExpandedDataCoordinate` only.
@@ -158,7 +153,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
        instance for at least one operand in any data ID comparison is strongly
        recommended.
 
-    See also
+    See Also
     --------
     :ref:`lsst.daf.butler-dimensions_data_ids`
     """
@@ -169,11 +164,11 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
 
     @staticmethod
     def standardize(
-        mapping: Optional[NameLookupMapping[Dimension, DataIdValue]] = None,
+        mapping: NameLookupMapping[Dimension, DataIdValue] | None = None,
         *,
-        graph: Optional[DimensionGraph] = None,
-        universe: Optional[DimensionUniverse] = None,
-        defaults: Optional[DataCoordinate] = None,
+        graph: DimensionGraph | None = None,
+        universe: DimensionUniverse | None = None,
+        defaults: DataCoordinate | None = None,
         **kwargs: Any,
     ) -> DataCoordinate:
         """Standardize the supplied dataId.
@@ -214,7 +209,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         KeyError
             Raised if a key-value pair for a required dimension is missing.
         """
-        d: Dict[str, DataIdValue] = {}
+        d: dict[str, DataIdValue] = {}
         if isinstance(mapping, DataCoordinate):
             if graph is None:
                 if not kwargs:
@@ -253,7 +248,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
                 for k, v in defaults.items():
                     d.setdefault(k.name, v)
         if d.keys() >= graph.dimensions.names:
-            values = tuple(d[name] for name in graph._dataCoordinateIndices.keys())
+            values = tuple(d[name] for name in graph._dataCoordinateIndices)
         else:
             try:
                 values = tuple(d[name] for name in graph.required.names)
@@ -287,7 +282,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         return _ExpandedTupleDataCoordinate(universe.empty, (), {})
 
     @staticmethod
-    def fromRequiredValues(graph: DimensionGraph, values: Tuple[DataIdValue, ...]) -> DataCoordinate:
+    def fromRequiredValues(graph: DimensionGraph, values: tuple[DataIdValue, ...]) -> DataCoordinate:
         """Construct a `DataCoordinate` from required dimension values.
 
         This is a low-level interface with at most assertion-level checking of
@@ -315,7 +310,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         return _BasicTupleDataCoordinate(graph, values)
 
     @staticmethod
-    def fromFullValues(graph: DimensionGraph, values: Tuple[DataIdValue, ...]) -> DataCoordinate:
+    def fromFullValues(graph: DimensionGraph, values: tuple[DataIdValue, ...]) -> DataCoordinate:
         """Construct a `DataCoordinate` from all dimension values.
 
         This is a low-level interface with at most assertion-level checking of
@@ -345,12 +340,12 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         return _BasicTupleDataCoordinate(graph, values)
 
     def __hash__(self) -> int:
-        return hash((self.graph,) + tuple(self[d.name] for d in self.graph.required))
+        return hash((self.graph,) + self.values_tuple())
 
     def __eq__(self, other: Any) -> bool:
         if not isinstance(other, DataCoordinate):
             other = DataCoordinate.standardize(other, universe=self.universe)
-        return self.graph == other.graph and all(self[d.name] == other[d.name] for d in self.graph.required)
+        return self.graph == other.graph and self.values_tuple() == other.values_tuple()
 
     def __repr__(self) -> str:
         # We can't make repr yield something that could be exec'd here without
@@ -385,7 +380,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         return self.graph.required
 
     @property
-    def names(self) -> AbstractSet[str]:
+    def names(self) -> Set[str]:
         """Names of the required dimensions identified by this data ID.
 
         They are returned in the same order as `keys`
@@ -459,7 +454,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
 
     @abstractmethod
     def expanded(
-        self, records: NameLookupMapping[DimensionElement, Optional[DimensionRecord]]
+        self, records: NameLookupMapping[DimensionElement, DimensionRecord | None]
     ) -> DataCoordinate:
         """Return a `DataCoordinate` that holds the given records.
 
@@ -470,10 +465,11 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
 
         Parameters
         ----------
-        records : `Mapping` [ `str`, `DimensionRecord` or `None` ]
+        records : `~collections.abc.Mapping` [ `str`, `DimensionRecord` or \
+                `None` ]
             A `NamedKeyMapping` with `DimensionElement` keys or a regular
-            `Mapping` with `str` (`DimensionElement` name) keys and
-            `DimensionRecord` values.  Keys must cover all elements in
+            `~collections.abc.Mapping` with `str` (`DimensionElement` name)
+            keys and `DimensionRecord` values.  Keys must cover all elements in
             ``self.graph.elements``.  Values may be `None`, but only to reflect
             actual NULL values in the database, not just records that have not
             been fetched.
@@ -517,6 +513,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         raise NotImplementedError()
 
     @property
+    @abstractmethod
     def full(self) -> NamedKeyMapping[Dimension, DataIdValue]:
         """Return mapping for all dimensions in ``self.graph``.
 
@@ -528,8 +525,17 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         when implied keys are accessed via the returned mapping, depending on
         the implementation and whether assertions are enabled.
         """
-        assert self.hasFull(), "full may only be accessed if hasFull() returns True."
-        return _DataCoordinateFullView(self)
+        raise NotImplementedError()
+
+    @abstractmethod
+    def values_tuple(self) -> tuple[DataIdValue, ...]:
+        """Return the required values (only) of this data ID as a tuple.
+
+        In contexts where all data IDs have the same dimensions, comparing and
+        hashing these tuples can be *much* faster than comparing the original
+        `DataCoordinate` instances.
+        """
+        raise NotImplementedError()
 
     @abstractmethod
     def hasRecords(self) -> bool:
@@ -552,7 +558,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         raise NotImplementedError()
 
     @property
-    def records(self) -> NamedKeyMapping[DimensionElement, Optional[DimensionRecord]]:
+    def records(self) -> NamedKeyMapping[DimensionElement, DimensionRecord | None]:
         """Return the records.
 
         Returns a  mapping that contains `DimensionRecord` objects for all
@@ -571,7 +577,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         return _DataCoordinateRecordsView(self)
 
     @abstractmethod
-    def _record(self, name: str) -> Optional[DimensionRecord]:
+    def _record(self, name: str) -> DimensionRecord | None:
         """Protected implementation hook that backs the ``records`` attribute.
 
         Parameters
@@ -589,7 +595,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         raise NotImplementedError()
 
     @property
-    def region(self) -> Optional[Region]:
+    def region(self) -> Region | None:
         """Spatial region associated with this data ID.
 
         (`lsst.sphgeom.Region` or `None`).
@@ -612,7 +618,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         return _intersectRegions(*regions)
 
     @property
-    def timespan(self) -> Optional[Timespan]:
+    def timespan(self) -> Timespan | None:
         """Temporal interval associated with this data ID.
 
         (`Timespan` or `None`).
@@ -642,14 +648,20 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
             return Timespan.intersection(*timespans)
 
     @overload
-    def pack(self, name: str, *, returnMaxBits: Literal[True]) -> Tuple[int, int]:
+    def pack(self, name: str, *, returnMaxBits: Literal[True]) -> tuple[int, int]:
         ...
 
     @overload
     def pack(self, name: str, *, returnMaxBits: Literal[False]) -> int:
         ...
 
-    def pack(self, name: str, *, returnMaxBits: bool = False) -> Union[Tuple[int, int], int]:
+    # TODO: Remove this method and its overloads above on DM-38687.
+    @deprecated(
+        "Deprecated in favor of configurable dimension packers.  Will be removed after v26.",
+        version="v26",
+        category=FutureWarning,
+    )
+    def pack(self, name: str, *, returnMaxBits: bool = False) -> tuple[int, int] | int:
         """Pack this data ID into an integer.
 
         Parameters
@@ -699,7 +711,7 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
             dataId = self.full.byName()
         else:
             dataId = self.byName()
-        records: Optional[Dict[str, SerializedDimensionRecord]]
+        records: dict[str, SerializedDimensionRecord] | None
         if not minimal and self.hasRecords():
             records = {k: v.to_simple() for k, v in self.records.byName().items() if v is not None}
         else:
@@ -711,8 +723,8 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
     def from_simple(
         cls,
         simple: SerializedDataCoordinate,
-        universe: Optional[DimensionUniverse] = None,
-        registry: Optional[Registry] = None,
+        universe: DimensionUniverse | None = None,
+        registry: Registry | None = None,
     ) -> DataCoordinate:
         """Construct a new object from the simplified form.
 
@@ -734,6 +746,10 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
         dataId : `DataCoordinate`
             Newly-constructed object.
         """
+        key = (frozenset(simple.dataId.items()), simple.records is not None)
+        cache = PersistenceContextVars.dataCoordinates.get()
+        if cache is not None and (result := cache.get(key)) is not None:
+            return result
         if universe is None and registry is None:
             raise ValueError("One of universe or registry is required to convert a dict to a DataCoordinate")
         if universe is None and registry is not None:
@@ -747,13 +763,15 @@ class DataCoordinate(NamedKeyMapping[Dimension, DataIdValue]):
             dataId = dataId.expanded(
                 {k: DimensionRecord.from_simple(v, universe=universe) for k, v in simple.records.items()}
             )
+        if cache is not None:
+            cache[key] = dataId
         return dataId
 
     to_json = to_json_pydantic
     from_json: ClassVar = classmethod(from_json_pydantic)
 
 
-DataId = Union[DataCoordinate, Mapping[str, Any]]
+DataId = DataCoordinate | Mapping[str, Any]
 """A type-annotation alias for signatures that accept both informal data ID
 dictionaries and validated `DataCoordinate` instances.
 """
@@ -771,7 +789,7 @@ class _DataCoordinateFullView(NamedKeyMapping[Dimension, DataIdValue]):
         The `DataCoordinate` instance this object provides a view of.
     """
 
-    def __init__(self, target: DataCoordinate):
+    def __init__(self, target: _BasicTupleDataCoordinate):
         self._target = target
 
     __slots__ = ("_target",)
@@ -793,12 +811,12 @@ class _DataCoordinateFullView(NamedKeyMapping[Dimension, DataIdValue]):
         return self._target.graph.dimensions
 
     @property
-    def names(self) -> AbstractSet[str]:
+    def names(self) -> Set[str]:
         # Docstring inherited from `NamedKeyMapping`.
         return self.keys().names
 
 
-class _DataCoordinateRecordsView(NamedKeyMapping[DimensionElement, Optional[DimensionRecord]]):
+class _DataCoordinateRecordsView(NamedKeyMapping[DimensionElement, DimensionRecord | None]):
     """View class for `DataCoordinate.records`.
 
     Provides the default implementation for
@@ -822,7 +840,7 @@ class _DataCoordinateRecordsView(NamedKeyMapping[DimensionElement, Optional[Dime
     def __str__(self) -> str:
         return "\n".join(str(v) for v in self.values())
 
-    def __getitem__(self, key: Union[DimensionElement, str]) -> Optional[DimensionRecord]:
+    def __getitem__(self, key: DimensionElement | str) -> DimensionRecord | None:
         if isinstance(key, DimensionElement):
             key = key.name
         return self._target._record(key)
@@ -837,7 +855,7 @@ class _DataCoordinateRecordsView(NamedKeyMapping[DimensionElement, Optional[Dime
         return self._target.graph.elements
 
     @property
-    def names(self) -> AbstractSet[str]:
+    def names(self) -> Set[str]:
         # Docstring inherited from `NamedKeyMapping`.
         return self.keys().names
 
@@ -861,7 +879,7 @@ class _BasicTupleDataCoordinate(DataCoordinate):
         or all dimensions.
     """
 
-    def __init__(self, graph: DimensionGraph, values: Tuple[DataIdValue, ...]):
+    def __init__(self, graph: DimensionGraph, values: tuple[DataIdValue, ...]):
         self._graph = graph
         self._values = values
 
@@ -884,6 +902,13 @@ class _BasicTupleDataCoordinate(DataCoordinate):
             # values for the required ones.
             raise KeyError(key) from None
 
+    def byName(self) -> dict[str, DataIdValue]:
+        # Docstring inheritance.
+        # Reimplementation is for optimization; `values_tuple()` is much faster
+        # to iterate over than values() because it doesn't go through
+        # `__getitem__`.
+        return dict(zip(self.names, self.values_tuple(), strict=True))
+
     def subset(self, graph: DimensionGraph) -> DataCoordinate:
         # Docstring inherited from DataCoordinate.
         if self._graph == graph:
@@ -891,7 +916,7 @@ class _BasicTupleDataCoordinate(DataCoordinate):
         elif self.hasFull() or self._graph.required >= graph.dimensions:
             return _BasicTupleDataCoordinate(
                 graph,
-                tuple(self[k] for k in graph._dataCoordinateIndices.keys()),
+                tuple(self[k] for k in graph._dataCoordinateIndices),
             )
         else:
             return _BasicTupleDataCoordinate(graph, tuple(self[k] for k in graph.required.names))
@@ -916,18 +941,23 @@ class _BasicTupleDataCoordinate(DataCoordinate):
             # There's some chance that neither self nor other has full values,
             # but together provide enough to the union to.  Let the general
             # case below handle that.
-        elif self.graph == graph:
+        elif self.graph == graph and self.hasFull():
             # No chance at returning records.  If self has full values, it's
             # the best we can do.
-            if self.hasFull():
-                return self
+            return self
         # General case with actual merging of dictionaries.
         values = self.full.byName() if self.hasFull() else self.byName()
         values.update(other.full.byName() if other.hasFull() else other.byName())
         return DataCoordinate.standardize(values, graph=graph)
 
+    @property
+    def full(self) -> NamedKeyMapping[Dimension, DataIdValue]:
+        # Docstring inherited.
+        assert self.hasFull(), "full may only be accessed if hasFull() returns True."
+        return _DataCoordinateFullView(self)
+
     def expanded(
-        self, records: NameLookupMapping[DimensionElement, Optional[DimensionRecord]]
+        self, records: NameLookupMapping[DimensionElement, DimensionRecord | None]
     ) -> DataCoordinate:
         # Docstring inherited from DataCoordinate
         values = self._values
@@ -947,9 +977,13 @@ class _BasicTupleDataCoordinate(DataCoordinate):
         # Docstring inherited from DataCoordinate.
         return False
 
-    def _record(self, name: str) -> Optional[DimensionRecord]:
+    def values_tuple(self) -> tuple[DataIdValue, ...]:
         # Docstring inherited from DataCoordinate.
-        assert False
+        return self._values[: len(self._graph.required)]
+
+    def _record(self, name: str) -> DimensionRecord | None:
+        # Docstring inherited from DataCoordinate.
+        raise AssertionError()
 
     def __reduce__(self) -> tuple[Any, ...]:
         return (_BasicTupleDataCoordinate, (self._graph, self._values))
@@ -977,10 +1011,10 @@ class _ExpandedTupleDataCoordinate(_BasicTupleDataCoordinate):
         Data ID values, ordered to match ``graph._dataCoordinateIndices``.
         May include values for just required dimensions (which always come
         first) or all dimensions.
-    records : `Mapping` [ `str`, `DimensionRecord` or `None` ]
+    records : `~collections.abc.Mapping` [ `str`, `DimensionRecord` or `None` ]
         A `NamedKeyMapping` with `DimensionElement` keys or a regular
-        `Mapping` with `str` (`DimensionElement` name) keys and
-        `DimensionRecord` values.  Keys must cover all elements in
+        `~collections.abc.Mapping` with `str` (`DimensionElement` name) keys
+        and `DimensionRecord` values.  Keys must cover all elements in
         ``self.graph.elements``.  Values may be `None`, but only to reflect
         actual NULL values in the database, not just records that have not
         been fetched.
@@ -989,8 +1023,8 @@ class _ExpandedTupleDataCoordinate(_BasicTupleDataCoordinate):
     def __init__(
         self,
         graph: DimensionGraph,
-        values: Tuple[DataIdValue, ...],
-        records: NameLookupMapping[DimensionElement, Optional[DimensionRecord]],
+        values: tuple[DataIdValue, ...],
+        records: NameLookupMapping[DimensionElement, DimensionRecord | None],
     ):
         super().__init__(graph, values)
         assert super().hasFull(), "This implementation requires full dimension records."
@@ -1003,11 +1037,11 @@ class _ExpandedTupleDataCoordinate(_BasicTupleDataCoordinate):
         if self._graph == graph:
             return self
         return _ExpandedTupleDataCoordinate(
-            graph, tuple(self[k] for k in graph._dataCoordinateIndices.keys()), records=self._records
+            graph, tuple(self[k] for k in graph._dataCoordinateIndices), records=self._records
         )
 
     def expanded(
-        self, records: NameLookupMapping[DimensionElement, Optional[DimensionRecord]]
+        self, records: NameLookupMapping[DimensionElement, DimensionRecord | None]
     ) -> DataCoordinate:
         # Docstring inherited from DataCoordinate.
         return self
@@ -1021,11 +1055,10 @@ class _ExpandedTupleDataCoordinate(_BasicTupleDataCoordinate):
             # self has records, so even if other is also a valid result, it's
             # no better.
             return self
-        if other.graph == graph:
+        if other.graph == graph and other.hasFull():
             # If other has full values, and self does not identify some of
             # those, it's the base we can do.  It may have records, too.
-            if other.hasFull():
-                return other
+            return other
             # If other does not have full values, there's a chance self may
             # provide the values needed to complete it.  For example, self
             # could be {band} while other could be
@@ -1044,7 +1077,7 @@ class _ExpandedTupleDataCoordinate(_BasicTupleDataCoordinate):
             elements -= self.graph.elements.names
             elements -= other.graph.elements.names
             if not elements:
-                records = NamedKeyDict[DimensionElement, Optional[DimensionRecord]](self.records)
+                records = NamedKeyDict[DimensionElement, DimensionRecord | None](self.records)
                 records.update(other.records)
                 return basic.expanded(records.freeze())
         return basic
@@ -1057,7 +1090,7 @@ class _ExpandedTupleDataCoordinate(_BasicTupleDataCoordinate):
         # Docstring inherited from DataCoordinate.
         return True
 
-    def _record(self, name: str) -> Optional[DimensionRecord]:
+    def _record(self, name: str) -> DimensionRecord | None:
         # Docstring inherited from DataCoordinate.
         return self._records[name]
 
