@@ -23,13 +23,16 @@ from __future__ import annotations
 
 __all__ = ("DimensionRecord", "SerializedDimensionRecord")
 
-from typing import TYPE_CHECKING, Any, ClassVar, Dict, Optional, Tuple, Type, Union
+from collections.abc import Hashable
+from typing import TYPE_CHECKING, Any, ClassVar, Optional, Tuple
 
 import lsst.sphgeom
+from lsst.daf.butler._compat import PYDANTIC_V2, _BaseModelCompat
 from lsst.utils.classes import immutable
-from pydantic import BaseModel, Field, StrictBool, StrictFloat, StrictInt, StrictStr, create_model
+from pydantic import Field, StrictBool, StrictFloat, StrictInt, StrictStr, create_model
 
 from ..json import from_json_pydantic, to_json_pydantic
+from ..persistenceContext import PersistenceContextVars
 from ..timespan import Timespan, TimespanDatabaseRepresentation
 from ._elements import Dimension, DimensionElement
 
@@ -40,7 +43,7 @@ if TYPE_CHECKING:  # Imports needed only for type annotations; may be circular.
     from ._schema import DimensionElementFields
 
 
-def _reconstructDimensionRecord(definition: DimensionElement, mapping: Dict[str, Any]) -> DimensionRecord:
+def _reconstructDimensionRecord(definition: DimensionElement, mapping: dict[str, Any]) -> DimensionRecord:
     """Unpickle implementation for `DimensionRecord` subclasses.
 
     For internal use by `DimensionRecord`.
@@ -48,7 +51,7 @@ def _reconstructDimensionRecord(definition: DimensionElement, mapping: Dict[str,
     return definition.RecordClass(**mapping)
 
 
-def _subclassDimensionRecord(definition: DimensionElement) -> Type[DimensionRecord]:
+def _subclassDimensionRecord(definition: DimensionElement) -> type[DimensionRecord]:
     """Create a dynamic subclass of `DimensionRecord` for the given element.
 
     For internal use by `DimensionRecord`.
@@ -65,16 +68,16 @@ def _subclassDimensionRecord(definition: DimensionElement) -> Type[DimensionReco
     return type(definition.name + ".RecordClass", (DimensionRecord,), d)
 
 
-class SpecificSerializedDimensionRecord(BaseModel, extra="forbid"):
+class SpecificSerializedDimensionRecord(_BaseModelCompat, extra="forbid"):
     """Base model for a specific serialized record content."""
 
 
-_SIMPLE_RECORD_CLASS_CACHE: Dict[
-    Tuple[DimensionElement, DimensionUniverse], Type[SpecificSerializedDimensionRecord]
+_SIMPLE_RECORD_CLASS_CACHE: dict[
+    tuple[DimensionElement, DimensionUniverse], type[SpecificSerializedDimensionRecord]
 ] = {}
 
 
-def _createSimpleRecordSubclass(definition: DimensionElement) -> Type[SpecificSerializedDimensionRecord]:
+def _createSimpleRecordSubclass(definition: DimensionElement) -> type[SpecificSerializedDimensionRecord]:
     from ._schema import DimensionElementFields
 
     # Cache on the definition (which hashes as the name) and the
@@ -115,48 +118,66 @@ def _createSimpleRecordSubclass(definition: DimensionElement) -> Type[SpecificSe
     return model
 
 
-class SerializedDimensionRecord(BaseModel):
+# While supporting pydantic v1 and v2 keep this outside the model.
+_serialized_dimension_record_schema_extra = {
+    "examples": [
+        {
+            "definition": "detector",
+            "record": {
+                "instrument": "HSC",
+                "id": 72,
+                "full_name": "0_01",
+                "name_in_raft": "01",
+                "raft": "0",
+                "purpose": "SCIENCE",
+            },
+        }
+    ]
+}
+
+
+class SerializedDimensionRecord(_BaseModelCompat):
     """Simplified model for serializing a `DimensionRecord`."""
 
     definition: str = Field(
         ...,
         title="Name of dimension associated with this record.",
-        example="exposure",
+        examples=["exposure"],
     )
 
     # Use strict types to prevent casting
-    record: Dict[str, Union[None, StrictFloat, StrictStr, StrictBool, StrictInt, Tuple[int, int]]] = Field(
+    record: dict[str, None | StrictInt | StrictFloat | StrictStr | StrictBool | tuple[int, int]] = Field(
         ...,
         title="Dimension record keys and values.",
-        example={
-            "definition": "exposure",
-            "record": {"instrument": "LATISS", "exposure": 2021050300044, "obs_id": "AT_O_20210503_00044"},
-        },
-    )
-
-    class Config:
-        """Local configuration overrides for model."""
-
-        schema_extra = {
-            "example": {
-                "definition": "detector",
+        examples=[
+            {
+                "definition": "exposure",
                 "record": {
-                    "instrument": "HSC",
-                    "id": 72,
-                    "full_name": "0_01",
-                    "name_in_raft": "01",
-                    "raft": "0",
-                    "purpose": "SCIENCE",
+                    "instrument": "LATISS",
+                    "exposure": 2021050300044,
+                    "obs_id": "AT_O_20210503_00044",
                 },
             }
+        ],
+    )
+
+    if PYDANTIC_V2:
+        model_config = {
+            "json_schema_extra": _serialized_dimension_record_schema_extra,  # type: ignore[typeddict-item]
         }
+    else:
+
+        class Config:
+            """Local configuration overrides for model."""
+
+            schema_extra = _serialized_dimension_record_schema_extra
 
     @classmethod
     def direct(
         cls,
         *,
         definition: str,
-        record: Dict[str, Union[None, StrictFloat, StrictStr, StrictBool, StrictInt, Tuple[int, int]]],
+        record: dict[str, None | StrictFloat | StrictStr | StrictBool | StrictInt | tuple[int, int]],
     ) -> SerializedDimensionRecord:
         """Construct a `SerializedDimensionRecord` directly without validators.
 
@@ -166,17 +187,27 @@ class SerializedDimensionRecord(BaseModel):
 
         This method should only be called when the inputs are trusted.
         """
-        node = cls.construct(definition=definition, record=record)
-        node = SerializedDimensionRecord.__new__(cls)
-        setter = object.__setattr__
-        setter(node, "definition", definition)
         # This method requires tuples as values of the mapping, but JSON
         # readers will read things in as lists. Be kind and transparently
         # transform to tuples
-        setter(
-            node, "record", {k: v if type(v) != list else tuple(v) for k, v in record.items()}  # type: ignore
+        _recItems = {
+            k: v if type(v) != list else tuple(v) for k, v in record.items()  # type: ignore # noqa: E721
+        }
+
+        # Type ignore because the ternary statement seems to confuse mypy
+        # based on conflicting inferred types of v.
+        key = (
+            definition,
+            frozenset(_recItems.items()),
         )
-        setter(node, "__fields_set__", {"definition", "record"})
+        cache = PersistenceContextVars.serializedDimensionRecordMapping.get()
+        if cache is not None and (result := cache.get(key)) is not None:
+            return result
+
+        node = cls.model_construct(definition=definition, record=_recItems)  # type: ignore
+
+        if cache is not None:
+            cache[key] = node
         return node
 
 
@@ -242,16 +273,15 @@ class DimensionRecord:
                     )
         for name in self.__slots__:
             object.__setattr__(self, name, kwargs.get(name))
-        if self.definition.temporal is not None:
-            if self.timespan is None:
-                object.__setattr__(
-                    self,
-                    "timespan",
-                    Timespan(
-                        kwargs.get("datetime_begin"),
-                        kwargs.get("datetime_end"),
-                    ),
-                )
+        if self.definition.temporal is not None and self.timespan is None:
+            object.__setattr__(
+                self,
+                "timespan",
+                Timespan(
+                    kwargs.get("datetime_begin"),
+                    kwargs.get("datetime_end"),
+                ),
+            )
 
         from ._coordinate import DataCoordinate
 
@@ -312,6 +342,10 @@ class DimensionRecord:
         # query. This may not be overly useful since to reconstruct
         # a collection of records will require repeated registry queries.
         # For now do not implement minimal form.
+        key = (id(self.definition), self.dataId)
+        cache = PersistenceContextVars.serializedDimensionRecordMapping.get()
+        if cache is not None and (result := cache.get(key)) is not None:
+            return result
 
         mapping = {name: getattr(self, name) for name in self.__slots__}
         # If the item in mapping supports simplification update it
@@ -330,14 +364,18 @@ class DimensionRecord:
                     # hash objects, encode it here to a hex string
                     mapping[k] = v.hex()
         definition = self.definition.to_simple(minimal=minimal)
-        return SerializedDimensionRecord(definition=definition, record=mapping)
+        dimRec = SerializedDimensionRecord(definition=definition, record=mapping)
+        if cache is not None:
+            cache[key] = dimRec
+        return dimRec
 
     @classmethod
     def from_simple(
         cls,
         simple: SerializedDimensionRecord,
-        universe: Optional[DimensionUniverse] = None,
-        registry: Optional[Registry] = None,
+        universe: DimensionUniverse | None = None,
+        registry: Registry | None = None,
+        cacheKey: Hashable | None = None,
     ) -> DimensionRecord:
         """Construct a new object from the simplified form.
 
@@ -354,6 +392,10 @@ class DimensionRecord:
         registry : `lsst.daf.butler.Registry`, optional
             Registry from which a universe can be extracted. Can be `None`
             if universe is provided explicitly.
+        cacheKey : `Hashable` or `None`
+            If this is not None, it will be used as a key for any cached
+            reconstruction instead of calculating a value from the serialized
+            format.
 
         Returns
         -------
@@ -367,6 +409,15 @@ class DimensionRecord:
         if universe is None:
             # this is for mypy
             raise ValueError("Unable to determine a usable universe")
+        # Type ignore because the ternary statement seems to confuse mypy
+        # based on conflicting inferred types of v.
+        key = cacheKey or (
+            simple.definition,
+            frozenset(simple.record.items()),  # type: ignore
+        )
+        cache = PersistenceContextVars.dimensionRecords.get()
+        if cache is not None and (result := cache.get(key)) is not None:
+            return result
 
         definition = DimensionElement.from_simple(simple.definition, universe=universe)
 
@@ -379,7 +430,7 @@ class DimensionRecord:
 
         # Timespan and region have to be converted to native form
         # for now assume that those keys are special
-        rec = record_model.dict()
+        rec = record_model.model_dump()
 
         if (ts := "timespan") in rec:
             rec[ts] = Timespan.from_simple(rec[ts], universe=universe, registry=registry)
@@ -389,12 +440,15 @@ class DimensionRecord:
         if (hsh := "hash") in rec:
             rec[hsh] = bytes.fromhex(rec[hsh].decode())
 
-        return _reconstructDimensionRecord(definition, rec)
+        dimRec = _reconstructDimensionRecord(definition, rec)
+        if cache is not None:
+            cache[key] = dimRec
+        return dimRec
 
     to_json = to_json_pydantic
     from_json: ClassVar = classmethod(from_json_pydantic)
 
-    def toDict(self, splitTimespan: bool = False) -> Dict[str, Any]:
+    def toDict(self, splitTimespan: bool = False) -> dict[str, Any]:
         """Return a vanilla `dict` representation of this record.
 
         Parameters
