@@ -19,16 +19,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from __future__ import annotations
-
 """In-memory datastore."""
 
+from __future__ import annotations
+
 __all__ = ("StoredMemoryItemInfo", "InMemoryDatastore")
 
 import logging
 import time
+from collections.abc import Iterable, Mapping
 from dataclasses import dataclass
-from typing import TYPE_CHECKING, Any, Dict, Iterable, List, Mapping, Optional, Set, Tuple, Union
+from typing import TYPE_CHECKING, Any
 from urllib.parse import urlencode
 
 from lsst.daf.butler import (
@@ -43,6 +44,7 @@ from lsst.daf.butler.core.utils import transactional
 from lsst.daf.butler.registry.interfaces import DatastoreRegistryBridge
 from lsst.resources import ResourcePath
 
+from ..registry.interfaces import DatabaseInsertMode
 from .genericDatastore import GenericBaseDatastore
 
 if TYPE_CHECKING:
@@ -106,34 +108,34 @@ class InMemoryDatastore(GenericBaseDatastore):
     """A new datastore is created every time and datasets disappear when
     the process shuts down."""
 
-    datasets: Dict[DatasetId, Any]
+    datasets: dict[DatasetId, Any]
     """Internal storage of datasets indexed by dataset ID."""
 
-    records: Dict[DatasetId, StoredMemoryItemInfo]
+    records: dict[DatasetId, StoredMemoryItemInfo]
     """Internal records about stored datasets."""
 
     def __init__(
         self,
-        config: Union[Config, str],
+        config: Config | str,
         bridgeManager: DatastoreRegistryBridgeManager,
-        butlerRoot: Optional[str] = None,
+        butlerRoot: str | None = None,
     ):
         super().__init__(config, bridgeManager)
 
         # Name ourselves with the timestamp the datastore
         # was created.
-        self.name = "{}@{}".format(type(self).__name__, time.time())
+        self.name = f"{type(self).__name__}@{time.time()}"
         log.debug("Creating datastore %s", self.name)
 
         # Storage of datasets, keyed by dataset_id
-        self.datasets: Dict[DatasetId, Any] = {}
+        self.datasets: dict[DatasetId, Any] = {}
 
         # Records is distinct in order to track concrete composite components
         # where we register multiple components for a single dataset.
-        self.records: Dict[DatasetId, StoredMemoryItemInfo] = {}
+        self.records: dict[DatasetId, StoredMemoryItemInfo] = {}
 
         # Related records that share the same parent
-        self.related: Dict[DatasetId, Set[DatasetId]] = {}
+        self.related: dict[DatasetId, set[DatasetId]] = {}
 
         self._bridge = bridgeManager.register(self.name, ephemeral=True)
 
@@ -177,21 +179,22 @@ class InMemoryDatastore(GenericBaseDatastore):
         # Docstring inherited from GenericBaseDatastore.
         return self._bridge
 
-    def addStoredItemInfo(self, refs: Iterable[DatasetRef], infos: Iterable[StoredMemoryItemInfo]) -> None:
+    def addStoredItemInfo(
+        self,
+        refs: Iterable[DatasetRef],
+        infos: Iterable[StoredMemoryItemInfo],
+        insert_mode: DatabaseInsertMode = DatabaseInsertMode.INSERT,
+    ) -> None:
         # Docstring inherited from GenericBaseDatastore.
-        for ref, info in zip(refs, infos):
-            if ref.id is None:
-                raise RuntimeError(f"Can not store unresolved DatasetRef {ref}")
+        for ref, info in zip(refs, infos, strict=True):
             self.records[ref.id] = info
             self.related.setdefault(info.parentID, set()).add(ref.id)
 
     def getStoredItemInfo(self, ref: DatasetIdRef) -> StoredMemoryItemInfo:
         # Docstring inherited from GenericBaseDatastore.
-        if ref.id is None:
-            raise RuntimeError(f"Can not retrieve unresolved DatasetRef {ref}")
         return self.records[ref.id]
 
-    def getStoredItemsInfo(self, ref: DatasetIdRef) -> List[StoredMemoryItemInfo]:
+    def getStoredItemsInfo(self, ref: DatasetIdRef) -> list[StoredMemoryItemInfo]:
         # Docstring inherited from GenericBaseDatastore.
         return [self.getStoredItemInfo(ref)]
 
@@ -200,15 +203,13 @@ class InMemoryDatastore(GenericBaseDatastore):
         # If a component has been removed previously then we can sometimes
         # be asked to remove it again. Other datastores ignore this
         # so also ignore here
-        if ref.id is None:
-            raise RuntimeError(f"Can not remove unresolved DatasetRef {ref}")
         if ref.id not in self.records:
             return
         record = self.records[ref.id]
         del self.records[ref.id]
         self.related[record.parentID].remove(ref.id)
 
-    def _get_dataset_info(self, ref: DatasetIdRef) -> Tuple[DatasetId, StoredMemoryItemInfo]:
+    def _get_dataset_info(self, ref: DatasetIdRef) -> tuple[DatasetId, StoredMemoryItemInfo]:
         """Check that the dataset is present and return the real ID and
         associated information.
 
@@ -283,8 +284,8 @@ class InMemoryDatastore(GenericBaseDatastore):
     def get(
         self,
         ref: DatasetRef,
-        parameters: Optional[Mapping[str, Any]] = None,
-        storageClass: Optional[Union[StorageClass, str]] = None,
+        parameters: Mapping[str, Any] | None = None,
+        storageClass: StorageClass | str | None = None,
     ) -> Any:
         """Load an InMemoryDataset from the store.
 
@@ -316,7 +317,6 @@ class InMemoryDatastore(GenericBaseDatastore):
         ValueError
             Formatter failed to process the dataset.
         """
-
         log.debug("Retrieve %s from %s with parameters %s", ref, self.name, parameters)
 
         realID, storedItemInfo = self._get_dataset_info(ref)
@@ -391,10 +391,6 @@ class InMemoryDatastore(GenericBaseDatastore):
         allow `ChainedDatastore` to put to multiple datastores without
         requiring that every datastore accepts the dataset.
         """
-
-        if ref.id is None:
-            raise RuntimeError(f"Can not store unresolved DatasetRef {ref}")
-
         # May need to coerce the in memory dataset to the correct
         # python type, otherwise parameters may not work.
         inMemoryDataset = ref.datasetType.storageClass.coerce_type(inMemoryDataset)
@@ -408,7 +404,7 @@ class InMemoryDatastore(GenericBaseDatastore):
         # expire it. Instead of storing a filename here, we include the
         # ID of this datasetRef so we can find it from components.
         itemInfo = StoredMemoryItemInfo(
-            time.time(), ref.datasetType.storageClass, parentID=ref.id, dataset_id=ref.getCheckedId()
+            time.time(), ref.datasetType.storageClass, parentID=ref.id, dataset_id=ref.id
         )
 
         # We have to register this content with registry.
@@ -443,14 +439,13 @@ class InMemoryDatastore(GenericBaseDatastore):
         The URIs returned for in-memory datastores are not usable but
         provide an indication of the associated dataset.
         """
-
         # Include the dataID as a URI query
         query = urlencode(ref.dataId)
 
         # if this has never been written then we have to guess
         if not self.exists(ref):
             if not predict:
-                raise FileNotFoundError("Dataset {} not in this datastore".format(ref))
+                raise FileNotFoundError(f"Dataset {ref} not in this datastore")
             name = f"{ref.datasetType.name}"
             fragment = "#predicted"
         else:
@@ -505,8 +500,8 @@ class InMemoryDatastore(GenericBaseDatastore):
         destination: ResourcePath,
         transfer: str = "auto",
         preserve_path: bool = True,
-        overwrite: Optional[bool] = False,
-    ) -> List[ResourcePath]:
+        overwrite: bool | None = False,
+    ) -> list[ResourcePath]:
         """Retrieve the file artifacts associated with the supplied refs.
 
         Notes
@@ -525,7 +520,7 @@ class InMemoryDatastore(GenericBaseDatastore):
             self.removeStoredItemInfo(ref)
 
     @transactional
-    def trash(self, ref: Union[DatasetRef, Iterable[DatasetRef]], ignore_errors: bool = False) -> None:
+    def trash(self, ref: DatasetRef | Iterable[DatasetRef], ignore_errors: bool = False) -> None:
         """Indicate to the Datastore that a dataset can be removed.
 
         Parameters
@@ -619,7 +614,7 @@ class InMemoryDatastore(GenericBaseDatastore):
                 self.removeStoredItemInfo(ref)
 
     def validateConfiguration(
-        self, entities: Iterable[Union[DatasetRef, DatasetType, StorageClass]], logFailures: bool = False
+        self, entities: Iterable[DatasetRef | DatasetType | StorageClass], logFailures: bool = False
     ) -> None:
         """Validate some of the configuration for this datastore.
 
@@ -644,22 +639,22 @@ class InMemoryDatastore(GenericBaseDatastore):
         """
         return
 
-    def _overrideTransferMode(self, *datasets: Any, transfer: Optional[str] = None) -> Optional[str]:
+    def _overrideTransferMode(self, *datasets: Any, transfer: str | None = None) -> str | None:
         # Docstring is inherited from base class
         return transfer
 
-    def validateKey(self, lookupKey: LookupKey, entity: Union[DatasetRef, DatasetType, StorageClass]) -> None:
+    def validateKey(self, lookupKey: LookupKey, entity: DatasetRef | DatasetType | StorageClass) -> None:
         # Docstring is inherited from base class
         return
 
-    def getLookupKeys(self) -> Set[LookupKey]:
+    def getLookupKeys(self) -> set[LookupKey]:
         # Docstring is inherited from base class
         return self.constraints.getLookupKeys()
 
     def needs_expanded_data_ids(
         self,
-        transfer: Optional[str],
-        entity: Optional[Union[DatasetRef, DatasetType, StorageClass]] = None,
+        transfer: str | None,
+        entity: DatasetRef | DatasetType | StorageClass | None = None,
     ) -> bool:
         # Docstring inherited.
         return False
