@@ -24,8 +24,8 @@ from __future__ import annotations
 __all__ = ("RemoteRegistry",)
 
 import contextlib
-import functools
-from typing import TYPE_CHECKING, Any, Iterable, Iterator, Mapping, Sequence, Set
+from collections.abc import Iterable, Iterator, Mapping, Sequence
+from typing import TYPE_CHECKING, Any
 
 import httpx
 from lsst.daf.butler import __version__
@@ -40,7 +40,6 @@ from ..core import (
     DataId,
     DatasetAssociation,
     DatasetId,
-    DatasetIdFactory,
     DatasetIdGenEnum,
     DatasetRef,
     DatasetType,
@@ -65,14 +64,15 @@ from ..core.serverModels import (
     QueryDatasetsModel,
     QueryDimensionRecordsModel,
 )
-from ..registry import CollectionSummary, CollectionType, Registry, RegistryConfig, RegistryDefaults
+from ..registry import CollectionSummary, CollectionType, RegistryConfig, RegistryDefaults, _ButlerRegistry
 
 if TYPE_CHECKING:
     from .._butlerConfig import ButlerConfig
+    from ..registry._registry import CollectionArgType
     from ..registry.interfaces import CollectionRecord, DatastoreRegistryBridgeManager
 
 
-class RemoteRegistry(Registry):
+class RemoteRegistry(_ButlerRegistry):
     """Registry that can talk to a remote Butler server.
 
     Parameters
@@ -90,8 +90,8 @@ class RemoteRegistry(Registry):
         config: RegistryConfig | str | None = None,
         dimensionConfig: DimensionConfig | str | None = None,
         butlerRoot: ResourcePathExpression | None = None,
-    ) -> Registry:
-        """Create registry database and return `Registry` instance.
+    ) -> _ButlerRegistry:
+        """Create registry database and return `_ButlerRegistry` instance.
 
         A remote registry can not create a registry database. Calling this
         method will raise an exception.
@@ -105,7 +105,7 @@ class RemoteRegistry(Registry):
         butlerRoot: ResourcePathExpression | None = None,
         writeable: bool = True,
         defaults: RegistryDefaults | None = None,
-    ) -> Registry:
+    ) -> _ButlerRegistry:
         # Docstring inherited from lsst.daf.butler.registry.Registry
         config = cls.forceRegistryConfig(config)
         config.replaceRoot(butlerRoot)
@@ -131,10 +131,6 @@ class RemoteRegistry(Registry):
         self._db = server_uri
         self._defaults = defaults
 
-        # In the future DatasetIdFactory may become configurable and this
-        # instance will need to be shared with datasets manager.
-        self.datasetIdFactory = DatasetIdFactory()
-
         # All PUT calls should be short-circuited if not writeable.
         self._writeable = writeable
 
@@ -165,8 +161,8 @@ class RemoteRegistry(Registry):
         # Can be used to prevent any PUTs to server
         return self._writeable
 
-    def copy(self, defaults: RegistryDefaults | None = None) -> Registry:
-        # Docstring inherited from lsst.daf.butler.registry.Registry
+    def copy(self, defaults: RegistryDefaults | None = None) -> _ButlerRegistry:
+        # Docstring inherited from lsst.daf.butler.registry._ButlerRegistry
         if defaults is None:
             # No need to copy, because `RegistryDefaults` is immutable; we
             # effectively copy on write.
@@ -240,7 +236,6 @@ class RemoteRegistry(Registry):
         # Docstring inherited from lsst.daf.butler.registry.Registry
         raise NotImplementedError()
 
-    @functools.lru_cache
     def getCollectionType(self, name: str) -> CollectionType:
         # Docstring inherited from lsst.daf.butler.registry.Registry
         # This could use a local cache since collection types won't
@@ -275,7 +270,7 @@ class RemoteRegistry(Registry):
         # Docstring inherited from lsst.daf.butler.registry.Registry
         raise NotImplementedError()
 
-    def getCollectionParentChains(self, collection: str) -> Set[str]:
+    def getCollectionParentChains(self, collection: str) -> set[str]:
         # Docstring inherited from lsst.daf.butler.registry.Registry
         raise NotImplementedError()
 
@@ -335,7 +330,7 @@ class RemoteRegistry(Registry):
         datasetType: DatasetType | str,
         dataId: DataId | None = None,
         *,
-        collections: Any = None,
+        collections: CollectionArgType | None = None,
         timespan: Timespan | None = None,
         **kwargs: Any,
     ) -> DatasetRef | None:
@@ -357,8 +352,6 @@ class RemoteRegistry(Registry):
         self,
         datasets: Iterable[DatasetRef],
         expand: bool = True,
-        idGenerationMode: DatasetIdGenEnum = DatasetIdGenEnum.UNIQUE,
-        reuseIds: bool = False,
     ) -> list[DatasetRef]:
         # Docstring inherited from lsst.daf.butler.registry.Registry
         raise NotImplementedError()
@@ -455,7 +448,7 @@ class RemoteRegistry(Registry):
         self,
         expression: Any = ...,
         *,
-        components: bool | None = None,
+        components: bool | None = False,
         missing: list[str] | None = None,
     ) -> Iterable[DatasetType]:
         # Docstring inherited from lsst.daf.butler.registry.Registry
@@ -524,12 +517,12 @@ class RemoteRegistry(Registry):
         self,
         datasetType: Any,
         *,
-        collections: Any = None,
+        collections: CollectionArgType | None = None,
         dimensions: Iterable[Dimension | str] | None = None,
         dataId: DataId | None = None,
         where: str = "",
         findFirst: bool = False,
-        components: bool | None = None,
+        components: bool | None = False,
         bind: Mapping[str, Any] | None = None,
         check: bool = True,
         **kwargs: Any,
@@ -538,12 +531,13 @@ class RemoteRegistry(Registry):
         if dimensions is not None:
             dimensions = [str(d) for d in ensure_iterable(dimensions)]
 
+        collections_param: ExpressionQueryParameter | None = None
         if collections is not None:
-            collections = ExpressionQueryParameter.from_expression(collections)
+            collections_param = ExpressionQueryParameter.from_expression(collections)
 
         parameters = QueryDatasetsModel(
             datasetType=ExpressionQueryParameter.from_expression(datasetType),
-            collections=collections,
+            collections=collections_param,
             dimensions=dimensions,
             dataId=self._simplify_dataId(dataId),
             where=where,
@@ -556,7 +550,7 @@ class RemoteRegistry(Registry):
 
         response = self._client.post(
             self._get_url("registry/datasets"),
-            json=parameters.dict(exclude_unset=True, exclude_defaults=True),
+            json=parameters.model_dump(mode="json", exclude_unset=True, exclude_defaults=True),
             timeout=20,
         )
         response.raise_for_status()
@@ -572,9 +566,9 @@ class RemoteRegistry(Registry):
         *,
         dataId: DataId | None = None,
         datasets: Any = None,
-        collections: Any = None,
+        collections: CollectionArgType | None = None,
         where: str = "",
-        components: bool | None = None,
+        components: bool | None = False,
         bind: Mapping[str, Any] | None = None,
         check: bool = True,
         **kwargs: Any,
@@ -582,15 +576,16 @@ class RemoteRegistry(Registry):
         # Docstring inherited from lsst.daf.butler.registry.Registry
         cleaned_dimensions = [str(d) for d in ensure_iterable(dimensions)]
 
+        collections_param: ExpressionQueryParameter | None = None
         if collections is not None:
-            collections = ExpressionQueryParameter.from_expression(collections)
+            collections_param = ExpressionQueryParameter.from_expression(collections)
         if datasets is not None:
             datasets = DatasetsQueryParameter.from_expression(datasets)
 
         parameters = QueryDataIdsModel(
             dimensions=cleaned_dimensions,
             dataId=self._simplify_dataId(dataId),
-            collections=collections,
+            collections=collections_param,
             datasets=datasets,
             where=where,
             components=components,
@@ -601,7 +596,7 @@ class RemoteRegistry(Registry):
 
         response = self._client.post(
             self._get_url("registry/dataIds"),
-            json=parameters.dict(exclude_unset=True, exclude_defaults=True),
+            json=parameters.model_dump(mode="json", exclude_unset=True, exclude_defaults=True),
             timeout=20,
         )
         response.raise_for_status()
@@ -621,23 +616,24 @@ class RemoteRegistry(Registry):
         *,
         dataId: DataId | None = None,
         datasets: Any = None,
-        collections: Any = None,
+        collections: CollectionArgType | None = None,
         where: str = "",
-        components: bool | None = None,
+        components: bool | None = False,
         bind: Mapping[str, Any] | None = None,
         check: bool = True,
         **kwargs: Any,
     ) -> Iterator[DimensionRecord]:
         # Docstring inherited from lsst.daf.butler.registry.Registry
+        collections_param: ExpressionQueryParameter | None = None
         if collections is not None:
-            collections = ExpressionQueryParameter.from_expression(collections)
+            collections_param = ExpressionQueryParameter.from_expression(collections)
         if datasets is not None:
             datasets = DatasetsQueryParameter.from_expression(datasets)
 
         parameters = QueryDimensionRecordsModel(
             dataId=self._simplify_dataId(dataId),
             datasets=datasets,
-            collections=collections,
+            collections=collections_param,
             where=where,
             components=components,
             bind=bind,
@@ -646,7 +642,7 @@ class RemoteRegistry(Registry):
         )
         response = self._client.post(
             self._get_url(f"registry/dimensionRecords/{element}"),
-            json=parameters.dict(exclude_unset=True, exclude_defaults=True),
+            json=parameters.model_dump(mode="json", exclude_unset=True, exclude_defaults=True),
             timeout=20,
         )
         response.raise_for_status()
@@ -661,7 +657,7 @@ class RemoteRegistry(Registry):
     def queryDatasetAssociations(
         self,
         datasetType: str | DatasetType,
-        collections: Any = ...,
+        collections: CollectionArgType | None = ...,
         *,
         collectionTypes: Iterable[CollectionType] = CollectionType.all(),
         flattenChains: bool = False,
