@@ -27,9 +27,10 @@ import itertools
 import warnings
 from abc import ABC, abstractmethod
 from collections import namedtuple
+from collections.abc import Iterable
 from concurrent.futures import ThreadPoolExecutor
-from contextlib import contextmanager
-from typing import Any, ContextManager, Iterable, Optional, Set, Tuple
+from contextlib import AbstractContextManager, contextmanager
+from typing import Any
 
 import astropy.time
 import sqlalchemy
@@ -93,7 +94,7 @@ def _patch_getExistingTable(db: Database) -> Database:
     """
     original_method = db.getExistingTable
 
-    def _getExistingTable(name: str, spec: ddl.TableSpec) -> Optional[sqlalchemy.schema.Table]:
+    def _getExistingTable(name: str, spec: ddl.TableSpec) -> sqlalchemy.schema.Table | None:
         # Return None on first call, but forward to original method after that
         db.getExistingTable = original_method
         return None
@@ -116,7 +117,7 @@ class DatabaseTests(ABC):
         raise NotImplementedError()
 
     @abstractmethod
-    def asReadOnly(self, database: Database) -> ContextManager[Database]:
+    def asReadOnly(self, database: Database) -> AbstractContextManager[Database]:
         """Return a context manager for a read-only connection into the given
         database.
 
@@ -144,9 +145,8 @@ class DatabaseTests(ABC):
         context-manager boilerplate that is usefully verbose in production code
         but just noise in tests.
         """
-        with database.transaction():
-            with database.query(executable) as result:
-                return result.fetchall()
+        with database.transaction(), database.query(executable) as result:
+            return result.fetchall()
 
     def query_scalar(self, database: Database, executable: sqlalchemy.sql.expression.SelectBase) -> Any:
         """Run a SELECT query that yields a single column and row against the
@@ -409,7 +409,7 @@ class DatabaseTests(ABC):
             r._asdict()
             for r in self.query_list(db, tables.b.select().where(tables.b.columns.id > results[1]["id"]))
         ]
-        expected = [dict(row, id=id) for row, id in zip(rows, ids)]
+        expected = [dict(row, id=id) for row, id in zip(rows, ids, strict=True)]
         self.assertCountEqual(results, expected)
         self.assertTrue(all(result["id"] is not None for result in results))
         # Insert multiple rows into a table with an autoincrement primary key,
@@ -417,7 +417,7 @@ class DatabaseTests(ABC):
         rows = [{"b_id": results[0]["id"]}, {"b_id": None}]
         ids = db.insert(tables.c, *rows, returnIds=True)
         results = [r._asdict() for r in self.query_list(db, tables.c.select())]
-        expected = [dict(row, id=id) for row, id in zip(rows, ids)]
+        expected = [dict(row, id=id) for row, id in zip(rows, ids, strict=True)]
         self.assertCountEqual(results, expected)
         self.assertTrue(all(result["id"] is not None for result in results))
         # Add the dynamic table.
@@ -728,7 +728,7 @@ class DatabaseTests(ABC):
         with db1.declareStaticTables(create=True) as context:
             tables1 = context.addTableTuple(STATIC_TABLE_SPECS)
 
-        async def side1(lock: Iterable[str] = ()) -> Tuple[Set[str], Set[str]]:
+        async def side1(lock: Iterable[str] = ()) -> tuple[set[str], set[str]]:
             """One side of the concurrent locking test.
 
             This optionally locks the table (and maybe the whole database),
@@ -750,7 +750,7 @@ class DatabaseTests(ABC):
             return names1, names2
 
         async def side2() -> None:
-            """The other side of the concurrent locking test.
+            """Other side of the concurrent locking test.
 
             This side just waits a bit and then tries to insert a row into the
             table that the other side is trying to lock.  Hopefully that
@@ -762,7 +762,9 @@ class DatabaseTests(ABC):
             """
 
             def toRunInThread():
-                """SQLite locking isn't asyncio-friendly unless we actually
+                """Create new SQLite connection for use in thread.
+
+                SQLite locking isn't asyncio-friendly unless we actually
                 run it in another thread.  And SQLite gets very unhappy if
                 we try to use a connection from multiple threads, so we have
                 to create the new connection here instead of out in the main
@@ -793,14 +795,16 @@ class DatabaseTests(ABC):
             await task2
             if "a2" in names1:
                 warnings.warn(
-                    "Unlucky scheduling in no-locking test: concurrent INSERT happened before first SELECT."
+                    "Unlucky scheduling in no-locking test: concurrent INSERT happened before first SELECT.",
+                    stacklevel=1,
                 )
                 self.assertEqual(names1, {"a2"})
                 self.assertEqual(names2, {"a1", "a2"})
             elif "a2" not in names2:
                 warnings.warn(
                     "Unlucky scheduling in no-locking test: concurrent INSERT "
-                    "happened after second SELECT even without locking."
+                    "happened after second SELECT even without locking.",
+                    stacklevel=1,
                 )
                 self.assertEqual(names1, set())
                 self.assertEqual(names2, {"a1"})
@@ -827,7 +831,8 @@ class DatabaseTests(ABC):
             await task2
             if "a2" in names1:
                 warnings.warn(
-                    "Unlucky scheduling in locking test: concurrent INSERT happened before first SELECT."
+                    "Unlucky scheduling in locking test: concurrent INSERT happened before first SELECT.",
+                    stacklevel=1,
                 )
                 self.assertEqual(names1, {"a2"})
                 self.assertEqual(names2, {"a1", "a2"})
@@ -860,7 +865,9 @@ class DatabaseTests(ABC):
         # Make another list of timespans that span the full range but don't
         # overlap.  This is a subset of the previous list.
         bTimespans = [Timespan(begin=None, end=timestamps[0])]
-        bTimespans.extend(Timespan(begin=t1, end=t2) for t1, t2 in zip(timestamps[:-1], timestamps[1:]))
+        bTimespans.extend(
+            Timespan(begin=t1, end=t2) for t1, t2 in zip(timestamps[:-1], timestamps[1:], strict=True)
+        )
         bTimespans.append(Timespan(begin=timestamps[-1], end=None))
         # Make a database and create a table with that database's timespan
         # representation.  This one will have no exclusion constraint and
