@@ -36,6 +36,7 @@ TESTDIR = os.path.abspath(os.path.dirname(__file__))
 
 
 def expectedFilesystemDatastoreTables(root: ResourcePath):
+    """Return the expected table contents."""
     return (
         AstropyTable(
             array(
@@ -134,6 +135,8 @@ def expectedFilesystemDatastoreTables(root: ResourcePath):
 
 
 class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
+    """Test the query-datasets command-line."""
+
     configFile = os.path.join(TESTDIR, "config/basic/butler.yaml")
     storageClassFactory = StorageClassFactory()
 
@@ -155,9 +158,13 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
 
         tables = self._queryDatasets(repo=self.repoDir, show_uri=True)
 
+        # Want second datastore root.
+        roots = testRepo.butler.get_datastore_roots()
+        datastore_root = roots[testRepo.butler.get_datastore_names()[1]]
+
         self.assertAstropyTablesEqual(
             tables,
-            expectedFilesystemDatastoreTables(testRepo.butler.datastore.datastores[1].root),
+            expectedFilesystemDatastoreTables(datastore_root),
             filterColumns=True,
         )
 
@@ -167,8 +174,11 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
 
         tables = self._queryDatasets(repo=self.repoDir, show_uri=True)
 
+        roots = testRepo.butler.get_datastore_roots()
+        datastore_root = list(roots.values())[0]
+
         self.assertAstropyTablesEqual(
-            tables, expectedFilesystemDatastoreTables(testRepo.butler.datastore.root), filterColumns=True
+            tables, expectedFilesystemDatastoreTables(datastore_root), filterColumns=True
         )
 
     def testNoShowURI(self):
@@ -238,7 +248,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                 names=("type", "run", "band", "instrument", "physical_filter", "visit"),
             ),
             AstropyTable(
-                array((("alt_test_metric_comp", "ingest/run", "R", "DummyCamComp", "d-r", "425"))),
+                array(("alt_test_metric_comp", "ingest/run", "R", "DummyCamComp", "d-r", "425")),
                 names=("type", "run", "band", "instrument", "physical_filter", "visit"),
             ),
         )
@@ -247,8 +257,8 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
 
     def testFindFirstAndCollections(self):
         """Test the find-first option, and the collections option, since it
-        is required for find-first."""
-
+        is required for find-first.
+        """
         testRepo = MetricTestRepo(self.repoDir, configFile=self.configFile)
 
         # Add a new run, and add a dataset to shadow an existing dataset.
@@ -257,6 +267,11 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
         # Verify that without find-first, duplicate datasets are returned
         tables = self._queryDatasets(repo=self.repoDir, collections=["foo", "ingest/run"], show_uri=True)
 
+        # The test should be running with a single FileDatastore.
+        roots = testRepo.butler.get_datastore_roots()
+        assert len(roots) == 1
+        datastore_root = list(roots.values())[0]
+
         expectedTables = (
             AstropyTable(
                 array(
@@ -268,7 +283,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "424",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "foo/test_metric_comp.data/test_metric_comp_v00000424_fDummyCamComp_data.yaml"
                             ),
                         ),
@@ -279,7 +294,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "423",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "ingest/run/test_metric_comp.data/"
                                 "test_metric_comp_v00000423_fDummyCamComp_data.yaml"
                             ),
@@ -291,7 +306,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "424",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "ingest/run/test_metric_comp.data/"
                                 "test_metric_comp_v00000424_fDummyCamComp_data.yaml"
                             ),
@@ -310,7 +325,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "424",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "foo/test_metric_comp.output/"
                                 "test_metric_comp_v00000424_fDummyCamComp_output.yaml"
                             ),
@@ -322,7 +337,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "423",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "ingest/run/test_metric_comp.output/"
                                 "test_metric_comp_v00000423_fDummyCamComp_output.yaml"
                             ),
@@ -334,7 +349,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "424",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "ingest/run/test_metric_comp.output/"
                                 "test_metric_comp_v00000424_fDummyCamComp_output.yaml"
                             ),
@@ -353,7 +368,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "424",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "foo/test_metric_comp.summary/"
                                 "test_metric_comp_v00000424_fDummyCamComp_summary.yaml"
                             ),
@@ -365,7 +380,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "423",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "ingest/run/test_metric_comp.summary/"
                                 "test_metric_comp_v00000423_fDummyCamComp_summary.yaml"
                             ),
@@ -377,7 +392,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "424",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "ingest/run/test_metric_comp.summary/"
                                 "test_metric_comp_v00000424_fDummyCamComp_summary.yaml"
                             ),
@@ -407,7 +422,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "424",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "foo/test_metric_comp.data/test_metric_comp_v00000424_fDummyCamComp_data.yaml"
                             ),
                         ),
@@ -418,7 +433,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "423",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "ingest/run/test_metric_comp.data/"
                                 "test_metric_comp_v00000423_fDummyCamComp_data.yaml"
                             ),
@@ -437,7 +452,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "424",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "foo/test_metric_comp.output/"
                                 "test_metric_comp_v00000424_fDummyCamComp_output.yaml"
                             ),
@@ -449,7 +464,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "423",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "ingest/run/test_metric_comp.output/"
                                 "test_metric_comp_v00000423_fDummyCamComp_output.yaml"
                             ),
@@ -468,7 +483,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "424",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "foo/test_metric_comp.summary/"
                                 "test_metric_comp_v00000424_fDummyCamComp_summary.yaml"
                             ),
@@ -480,7 +495,7 @@ class QueryDatasetsTest(unittest.TestCase, ButlerTestHelper):
                             "DummyCamComp",
                             "d-r",
                             "423",
-                            testRepo.butler.datastore.root.join(
+                            datastore_root.join(
                                 "ingest/run/test_metric_comp.summary/"
                                 "test_metric_comp_v00000423_fDummyCamComp_summary.yaml"
                             ),
