@@ -21,12 +21,13 @@
 
 import copy
 import itertools
+import math
 import os
 import pickle
 import unittest
+from collections.abc import Iterator
 from dataclasses import dataclass
 from random import Random
-from typing import Iterator, Optional
 
 import lsst.sphgeom
 from lsst.daf.butler import (
@@ -37,14 +38,14 @@ from lsst.daf.butler import (
     Dimension,
     DimensionConfig,
     DimensionGraph,
+    DimensionPacker,
     DimensionUniverse,
     NamedKeyDict,
     NamedValueSet,
-    Registry,
     TimespanDatabaseRepresentation,
     YamlRepoImportBackend,
 )
-from lsst.daf.butler.registry import RegistryConfig
+from lsst.daf.butler.registry import RegistryConfig, _RegistryFactory
 
 DIMENSION_DATA_FILE = os.path.normpath(
     os.path.join(os.path.dirname(__file__), "data", "registry", "hsc-rc2-subset.yaml")
@@ -63,8 +64,8 @@ def loadDimensionData() -> DataCoordinateSequence:
     # data and retreive it as a set of DataCoordinate objects.
     config = RegistryConfig()
     config["db"] = "sqlite://"
-    registry = Registry.createFromConfig(config)
-    with open(DIMENSION_DATA_FILE, "r") as stream:
+    registry = _RegistryFactory(config).create_from_config()
+    with open(DIMENSION_DATA_FILE) as stream:
         backend = YamlRepoImportBackend(stream, registry)
     backend.register()
     backend.load(datastore=None)
@@ -72,6 +73,37 @@ def loadDimensionData() -> DataCoordinateSequence:
     return registry.queryDataIds(dimensions).expanded().toSequence()
 
 
+class ConcreteTestDimensionPacker(DimensionPacker):
+    """A concrete `DimensionPacker` for testing its base class implementations.
+
+    This class just returns the detector ID as-is.
+    """
+
+    def __init__(self, fixed: DataCoordinate, dimensions: DimensionGraph):
+        super().__init__(fixed, dimensions)
+        self._n_detectors = fixed.records["instrument"].detector_max
+        self._max_bits = (self._n_detectors - 1).bit_length()
+
+    @property
+    def maxBits(self) -> int:
+        # Docstring inherited from DimensionPacker.maxBits
+        return self._max_bits
+
+    def _pack(self, dataId: DataCoordinate) -> int:
+        # Docstring inherited from DimensionPacker._pack
+        return dataId["detector"]
+
+    def unpack(self, packedId: int) -> DataCoordinate:
+        # Docstring inherited from DimensionPacker.unpack
+        return DataCoordinate.standardize(
+            {
+                "instrument": self.fixed["instrument"],
+                "detector": packedId,
+            },
+            graph=self.dimensions,
+        )
+
+
 class DimensionTestCase(unittest.TestCase):
     """Tests for dimensions.
 
@@ -253,6 +285,20 @@ class DimensionTestCase(unittest.TestCase):
         self.assertEqual(graph.temporal.names, {"observation_timespans"})
         self.assertEqual(next(iter(graph.spatial)).governor, self.universe["instrument"])
         self.assertEqual(next(iter(graph.temporal)).governor, self.universe["instrument"])
+        self.assertEqual(self.universe["visit_definition"].populated_by, self.universe["visit"])
+        self.assertEqual(self.universe["visit_system_membership"].populated_by, self.universe["visit"])
+        self.assertEqual(self.universe["visit_detector_region"].populated_by, self.universe["visit"])
+        self.assertEqual(
+            self.universe.get_elements_populated_by(self.universe["visit"]),
+            NamedValueSet(
+                {
+                    self.universe["visit"],
+                    self.universe["visit_definition"],
+                    self.universe["visit_system_membership"],
+                    self.universe["visit_detector_region"],
+                }
+            ),
+        )
 
     def testSkyMapDimensions(self):
         graph = DimensionGraph(self.universe, names=("patch",))
@@ -314,7 +360,7 @@ class DimensionTestCase(unittest.TestCase):
                 self.assertIn(foreignKey.table, tableSpecs)
                 self.assertIn(foreignKey.table, element.graph.dimensions.names)
                 self.assertEqual(len(foreignKey.source), len(foreignKey.target))
-                for source, target in zip(foreignKey.source, foreignKey.target):
+                for source, target in zip(foreignKey.source, foreignKey.target, strict=True):
                     self.assertIn(source, tableSpec.fields.names)
                     self.assertIn(target, tableSpecs[foreignKey.table].fields.names)
                     self.assertEqual(
@@ -351,7 +397,7 @@ class SplitByStateFlags:
     values.
     """
 
-    minimal: Optional[DataCoordinateSequence] = None
+    minimal: DataCoordinateSequence | None = None
     """Data IDs that only contain values for required dimensions.
 
     `DataCoordinateSequence.hasFull()` will return `True` for this if and only
@@ -359,21 +405,21 @@ class SplitByStateFlags:
     `DataCoordinate.hasRecords()` will always return `False`.
     """
 
-    complete: Optional[DataCoordinateSequence] = None
+    complete: DataCoordinateSequence | None = None
     """Data IDs that contain values for all dimensions.
 
     `DataCoordinateSequence.hasFull()` will always `True` and
     `DataCoordinate.hasRecords()` will always return `True` for this attribute.
     """
 
-    expanded: Optional[DataCoordinateSequence] = None
+    expanded: DataCoordinateSequence | None = None
     """Data IDs that contain values for all dimensions as well as records.
 
     `DataCoordinateSequence.hasFull()` and `DataCoordinate.hasRecords()` will
     always return `True` for this attribute.
     """
 
-    def chain(self, n: Optional[int] = None) -> Iterator:
+    def chain(self, n: int | None = None) -> Iterator:
         """Iterate over the data IDs of different types.
 
         Parameters
@@ -400,6 +446,8 @@ class SplitByStateFlags:
 
 
 class DataCoordinateTestCase(unittest.TestCase):
+    """Test `DataCoordinate`."""
+
     RANDOM_SEED = 10
 
     @classmethod
@@ -409,7 +457,7 @@ class DataCoordinateTestCase(unittest.TestCase):
     def setUp(self):
         self.rng = Random(self.RANDOM_SEED)
 
-    def randomDataIds(self, n: int, dataIds: Optional[DataCoordinateSequence] = None):
+    def randomDataIds(self, n: int, dataIds: DataCoordinateSequence | None = None):
         """Select random data IDs from those loaded from test data.
 
         Parameters
@@ -434,7 +482,7 @@ class DataCoordinateTestCase(unittest.TestCase):
             check=False,
         )
 
-    def randomDimensionSubset(self, n: int = 3, graph: Optional[DimensionGraph] = None) -> DimensionGraph:
+    def randomDimensionSubset(self, n: int = 3, graph: DimensionGraph | None = None) -> DimensionGraph:
         """Generate a random `DimensionGraph` that has a subset of the
         dimensions in a given one.
 
@@ -460,7 +508,7 @@ class DataCoordinateTestCase(unittest.TestCase):
 
     def splitByStateFlags(
         self,
-        dataIds: Optional[DataCoordinateSequence] = None,
+        dataIds: DataCoordinateSequence | None = None,
         *,
         expanded: bool = True,
         complete: bool = True,
@@ -518,14 +566,14 @@ class DataCoordinateTestCase(unittest.TestCase):
         applicable) its ``full`` property are self-consistent and consistent
         with the ``graph`` property.
         """
-        for n in range(5):
+        for _ in range(5):
             dimensions = self.randomDimensionSubset()
             dataIds = self.randomDataIds(n=1).subset(dimensions)
             split = self.splitByStateFlags(dataIds)
             for dataId in split.chain():
                 with self.subTest(dataId=dataId):
-                    self.assertEqual(list(dataId.values()), [dataId[d] for d in dataId.keys()])
-                    self.assertEqual(list(dataId.values()), [dataId[d.name] for d in dataId.keys()])
+                    self.assertEqual(list(dataId.values()), [dataId[d] for d in dataId])
+                    self.assertEqual(list(dataId.values()), [dataId[d.name] for d in dataId])
                     self.assertEqual(dataId.keys(), dataId.graph.required)
             for dataId in itertools.chain(split.complete, split.expanded):
                 with self.subTest(dataId=dataId):
@@ -534,7 +582,7 @@ class DataCoordinateTestCase(unittest.TestCase):
                     self.assertEqual(list(dataId.full.values()), [dataId[k] for k in dataId.graph.dimensions])
 
     def test_pickle(self):
-        for n in range(5):
+        for _ in range(5):
             dimensions = self.randomDimensionSubset()
             dataIds = self.randomDataIds(n=1).subset(dimensions)
             split = self.splitByStateFlags(dataIds)
@@ -553,7 +601,7 @@ class DataCoordinateTestCase(unittest.TestCase):
         """Test that dimension records are available as attributes on expanded
         data coordinates.
         """
-        for n in range(5):
+        for _ in range(5):
             dimensions = self.randomDimensionSubset()
             dataIds = self.randomDataIds(n=1).subset(dimensions)
             split = self.splitByStateFlags(dataIds)
@@ -562,13 +610,13 @@ class DataCoordinateTestCase(unittest.TestCase):
                     self.assertIs(getattr(data_id, element.name), data_id.records[element.name])
                     self.assertIn(element.name, dir(data_id))
                 with self.assertRaisesRegex(AttributeError, "^not_a_dimension_name$"):
-                    getattr(data_id, "not_a_dimension_name")
+                    data_id.not_a_dimension_name
             for data_id in itertools.chain(split.minimal, split.complete):
                 for element in data_id.graph.elements:
                     with self.assertRaisesRegex(AttributeError, "only available on expanded DataCoordinates"):
                         getattr(data_id, element.name)
                 with self.assertRaisesRegex(AttributeError, "^not_a_dimension_name$"):
-                    getattr(data_id, "not_a_dimension_name")
+                    data_id.not_a_dimension_name
 
     def testEquality(self):
         """Test that different `DataCoordinate` instances with different state
@@ -601,11 +649,11 @@ class DataCoordinateTestCase(unittest.TestCase):
         """Test constructing a DataCoordinate from many different kinds of
         input via `DataCoordinate.standardize` and `DataCoordinate.subset`.
         """
-        for n in range(5):
+        for _ in range(5):
             dimensions = self.randomDimensionSubset()
             dataIds = self.randomDataIds(n=1).subset(dimensions)
             split = self.splitByStateFlags(dataIds)
-            for m, dataId in enumerate(split.chain()):
+            for dataId in split.chain():
                 # Passing in any kind of DataCoordinate alone just returns
                 # that object.
                 self.assertIs(dataId, DataCoordinate.standardize(dataId))
@@ -711,9 +759,12 @@ class DataCoordinateTestCase(unittest.TestCase):
                             self.assertTrue(unioned.hasRecords())
                     if lhs.graph.required | rhs.graph.required >= unioned.graph.dimensions:
                         self.assertTrue(unioned.hasFull())
-                    if lhs.hasRecords() and rhs.hasRecords():
-                        if lhs.graph.elements | rhs.graph.elements >= unioned.graph.elements:
-                            self.assertTrue(unioned.hasRecords())
+                    if (
+                        lhs.hasRecords()
+                        and rhs.hasRecords()
+                        and lhs.graph.elements | rhs.graph.elements >= unioned.graph.elements
+                    ):
+                        self.assertTrue(unioned.hasRecords())
 
     def testRegions(self):
         """Test that data IDs for a few known dimensions have the expected
@@ -827,6 +878,22 @@ class DataCoordinateTestCase(unittest.TestCase):
         self.assertEqual(a ^ b, a.symmetric_difference(b))
         self.assertGreaterEqual(a ^ b, (a | b) - (a & b))
 
+    def testPackers(self):
+        (instrument_data_id,) = self.allDataIds.subset(
+            self.allDataIds.universe.extract(["instrument"])
+        ).toSet()
+        (detector_data_id,) = self.randomDataIds(n=1).subset(self.allDataIds.universe.extract(["detector"]))
+        packer = ConcreteTestDimensionPacker(instrument_data_id, detector_data_id.graph)
+        packed_id, max_bits = packer.pack(detector_data_id, returnMaxBits=True)
+        self.assertEqual(packed_id, detector_data_id["detector"])
+        self.assertEqual(max_bits, packer.maxBits)
+        self.assertEqual(
+            max_bits, math.ceil(math.log2(instrument_data_id.records["instrument"].detector_max))
+        )
+        self.assertEqual(packer.pack(detector_data_id), packed_id)
+        self.assertEqual(packer.pack(detector=detector_data_id["detector"]), detector_data_id["detector"])
+        self.assertEqual(packer.unpack(packed_id), detector_data_id)
+
 
 if __name__ == "__main__":
     unittest.main()
