@@ -25,7 +25,7 @@ import tempfile
 import unittest
 import warnings
 from abc import abstractmethod
-from typing import TYPE_CHECKING, Any, cast
+from typing import cast
 
 import astropy.time
 import sqlalchemy
@@ -33,13 +33,12 @@ from lsst.daf.butler import (
     CollectionType,
     Config,
     DataCoordinate,
-    DatasetIdGenEnum,
     DatasetRef,
     DatasetType,
     StorageClassFactory,
 )
 from lsst.daf.butler.registries.sql import SqlRegistry
-from lsst.daf.butler.registry import Registry, RegistryConfig
+from lsst.daf.butler.registry import Registry, RegistryConfig, _ButlerRegistry, _RegistryFactory
 from lsst.daf.butler.registry.obscore import (
     DatasetTypeConfig,
     ObsCoreConfig,
@@ -47,7 +46,7 @@ from lsst.daf.butler.registry.obscore import (
     ObsCoreSchema,
 )
 from lsst.daf.butler.registry.obscore._schema import _STATIC_COLUMNS
-from lsst.daf.butler.tests.utils import makeTestTempDir, removeTestTempDir
+from lsst.daf.butler.tests.utils import TestCaseMixin, makeTestTempDir, removeTestTempDir
 from lsst.sphgeom import Box, ConvexPolygon, LonLat, UnitVector3d
 
 try:
@@ -58,17 +57,17 @@ except ImportError:
 TESTDIR = os.path.abspath(os.path.dirname(__file__))
 
 
-class ObsCoreTests:
+class ObsCoreTests(TestCaseMixin):
     """Base class for testing obscore manager functionality."""
 
     root: str
 
     def make_registry(
         self, collections: list[str] | None = None, collection_type: str | None = None
-    ) -> Registry:
+    ) -> _ButlerRegistry:
         """Create new empty Registry."""
         config = self.make_registry_config(collections, collection_type)
-        registry = Registry.createFromConfig(config, butlerRoot=self.root)
+        registry = _RegistryFactory(config).create_from_config(butlerRoot=self.root)
         self.initialize_registry(registry)
         return registry
 
@@ -81,7 +80,6 @@ class ObsCoreTests:
 
     def initialize_registry(self, registry: Registry) -> None:
         """Populate Registry with the things that we need for tests."""
-
         registry.insertDimensionData("instrument", {"name": "DummyCam"})
         registry.insertDimensionData(
             "physical_filter", {"instrument": "DummyCam", "name": "d-r", "band": "r"}
@@ -226,10 +224,7 @@ class ObsCoreTests:
         coordinate = DataCoordinate.standardize(data_id, universe=registry.dimensions)
         if do_import:
             ds_type = self.dataset_types[dataset_type]
-            dataset_id = registry.datasetIdFactory.makeDatasetId(
-                run, ds_type, coordinate, DatasetIdGenEnum.UNIQUE
-            )
-            ref = DatasetRef(ds_type, coordinate, id=dataset_id, run=run)
+            ref = DatasetRef(ds_type, coordinate, run=run)
             [ref] = registry._importDatasets([ref])
         else:
             [ref] = registry.insertDatasets(dataset_type, [data_id], run=run)
@@ -250,7 +245,6 @@ class ObsCoreTests:
 
     def test_config_errors(self):
         """Test for handling various configuration problems."""
-
         # This raises pydantic ValidationError, which wraps ValueError
         exception_re = "'collections' must have one element"
         with self.assertRaisesRegex(ValueError, exception_re):
@@ -268,8 +262,7 @@ class ObsCoreTests:
 
     def test_schema(self):
         """Check how obscore schema is constructed"""
-
-        config = ObsCoreConfig(obs_collection="", dataset_types=[], facility_name="FACILITY")
+        config = ObsCoreConfig(obs_collection="", dataset_types={}, facility_name="FACILITY")
         schema = ObsCoreSchema(config, [])
         table_spec = schema.table_spec
         self.assertEqual(list(table_spec.fields.names), [col.name for col in _STATIC_COLUMNS])
@@ -278,7 +271,7 @@ class ObsCoreTests:
         config = ObsCoreConfig(
             obs_collection="",
             extra_columns={"c1": 1, "c2": "string", "c3": {"template": "{calib_level}", "type": "float"}},
-            dataset_types=[],
+            dataset_types={},
             facility_name="FACILITY",
         )
         schema = ObsCoreSchema(config, [])
@@ -351,7 +344,6 @@ class ObsCoreTests:
         """Test insert and import registry methods, with various restrictions
         on collection names.
         """
-
         # First item is collections, second item is expected record count.
         test_data = (
             (None, 6),
@@ -378,7 +370,6 @@ class ObsCoreTests:
 
     def test_drop_datasets(self):
         """Test for dropping datasets after obscore insert."""
-
         collections = None
         registry = self.make_registry(collections)
         obscore = registry.obsCoreTableManager
@@ -403,7 +394,6 @@ class ObsCoreTests:
 
     def test_associate(self):
         """Test for associating datasets to TAGGED collection."""
-
         collections = ["tagged"]
         registry = self.make_registry(collections, "TAGGED")
         obscore = registry.obsCoreTableManager
@@ -445,7 +435,6 @@ class ObsCoreTests:
 
     def test_region_type_warning(self) -> None:
         """Test that non-polygon region generates one or more warnings."""
-
         collections = None
         registry = self.make_registry(collections)
 
@@ -460,7 +449,6 @@ class ObsCoreTests:
 
     def test_update_exposure_region(self) -> None:
         """Test for update_exposure_regions method."""
-
         registry = self.make_registry(["run1"])
         obscore = registry.obsCoreTableManager
         assert obscore is not None
@@ -497,21 +485,6 @@ class ObsCoreTests:
                     self.assertIsNone(row.s_dec)
                     self.assertIsNone(row.s_region)
 
-    if TYPE_CHECKING:
-        # This is a mixin class, some methods from unittest.TestCase declared
-        # here to silence mypy.
-        def assertEqual(self, first: Any, second: Any, msg: str | None = None) -> None:
-            ...
-
-        def assertIsNone(self, obj: Any, msg: str | None = None) -> None:
-            ...
-
-        def assertIsNotNone(self, obj: Any, msg: str | None = None) -> None:
-            ...
-
-        def assertRegex(self, text: Any, expected_regex: Any, msg: str | None = None) -> None:
-            ...
-
 
 class SQLiteObsCoreTest(ObsCoreTests, unittest.TestCase):
     """Unit test for obscore with SQLite backend."""
@@ -599,7 +572,7 @@ class PostgresPgSphereObsCoreTest(PostgresObsCoreTest):
             try:
                 connection.execute(sqlalchemy.text("CREATE EXTENSION pg_sphere"))
             except sqlalchemy.exc.DatabaseError as exc:
-                raise unittest.SkipTest(f"pg_sphere extension does not exist: {exc}")
+                raise unittest.SkipTest(f"pg_sphere extension does not exist: {exc}") from None
 
     def make_obscore_config(
         self, collections: list[str] | None = None, collection_type: str | None = None
@@ -619,7 +592,6 @@ class PostgresPgSphereObsCoreTest(PostgresObsCoreTest):
 
     def test_spatial(self):
         """Test that pgsphere plugin fills spatial columns."""
-
         collections = None
         registry = self.make_registry(collections)
         obscore = registry.obsCoreTableManager
