@@ -43,7 +43,7 @@ except ImportError:
 try:
     import pandas as pd
 except ImportError:
-    np = None
+    pd = None
 
 from lsst.daf.butler import (
     Butler,
@@ -54,31 +54,52 @@ from lsst.daf.butler import (
     StorageClassConfig,
     StorageClassFactory,
 )
-from lsst.daf.butler.delegates.arrowastropy import ArrowAstropyDelegate
-from lsst.daf.butler.delegates.arrownumpy import ArrowNumpyDelegate
-from lsst.daf.butler.delegates.arrowtable import ArrowTableDelegate
-from lsst.daf.butler.delegates.dataframe import DataFrameDelegate
-from lsst.daf.butler.formatters.parquet import (
-    ArrowAstropySchema,
-    ArrowNumpySchema,
-    DataFrameSchema,
-    ParquetFormatter,
-    _append_numpy_multidim_metadata,
-    _astropy_to_numpy_dict,
-    _numpy_dict_to_numpy,
-    _numpy_dtype_to_arrow_types,
-    _numpy_style_arrays_to_arrow_arrays,
-    _numpy_to_numpy_dict,
-    arrow_to_astropy,
-    arrow_to_numpy,
-    arrow_to_numpy_dict,
-    arrow_to_pandas,
-    astropy_to_arrow,
-    compute_row_group_size,
-    numpy_dict_to_arrow,
-    numpy_to_arrow,
-    pandas_to_arrow,
-)
+
+try:
+    from lsst.daf.butler.delegates.arrowastropy import ArrowAstropyDelegate
+except ImportError:
+    atable = None
+    pa = None
+try:
+    from lsst.daf.butler.delegates.arrownumpy import ArrowNumpyDelegate
+except ImportError:
+    np = None
+    pa = None
+try:
+    from lsst.daf.butler.delegates.arrowtable import ArrowTableDelegate
+except ImportError:
+    pa = None
+try:
+    from lsst.daf.butler.delegates.dataframe import DataFrameDelegate
+except ImportError:
+    pd = None
+try:
+    from lsst.daf.butler.formatters.parquet import (
+        ArrowAstropySchema,
+        ArrowNumpySchema,
+        DataFrameSchema,
+        ParquetFormatter,
+        _append_numpy_multidim_metadata,
+        _astropy_to_numpy_dict,
+        _numpy_dict_to_numpy,
+        _numpy_dtype_to_arrow_types,
+        _numpy_style_arrays_to_arrow_arrays,
+        _numpy_to_numpy_dict,
+        arrow_to_astropy,
+        arrow_to_numpy,
+        arrow_to_numpy_dict,
+        arrow_to_pandas,
+        astropy_to_arrow,
+        compute_row_group_size,
+        numpy_dict_to_arrow,
+        numpy_to_arrow,
+        pandas_to_arrow,
+    )
+except ImportError:
+    pa = None
+    pd = None
+    atable = None
+    np = None
 from lsst.daf.butler.tests.utils import makeTestTempDir, removeTestTempDir
 
 TESTDIR = os.path.abspath(os.path.dirname(__file__))
@@ -145,13 +166,15 @@ def _makeSimpleNumpyTable(include_multidim=False, include_bigendian=False):
     return data
 
 
-def _makeSingleIndexDataFrame(include_masked=False):
+def _makeSingleIndexDataFrame(include_masked=False, include_lists=False):
     """Make a single index data frame for testing.
 
     Parameters
     ----------
     include_masked : `bool`
         Include masked columns.
+    include_lists : `bool`
+        Include list columns.
 
     Returns
     -------
@@ -172,6 +195,13 @@ def _makeSingleIndexDataFrame(include_masked=False):
         df["mstrcol"] = pd.array(np.array(["text"] * nrow))
         df.loc[1, ["m1", "m2", "mstrcol"]] = None
 
+    if include_lists:
+        nrow = len(df)
+
+        df["l1"] = [[0, 0]] * nrow
+        df["l2"] = [[0.0, 0.0]] * nrow
+        df["l3"] = [[]] * nrow
+
     allColumns = df.columns.append(pd.Index(df.index.names))
 
     return df, allColumns
@@ -267,11 +297,12 @@ class ParquetFormatterDataFrameTestCase(unittest.TestCase):
         """Create a new butler root for each test."""
         self.root = makeTestTempDir(TESTDIR)
         config = Config(self.configFile)
-        self.butler = Butler(Butler.makeRepo(self.root, config=config), writeable=True, run="test_run")
+        self.run = "test_run"
+        self.butler = Butler(Butler.makeRepo(self.root, config=config), writeable=True, run=self.run)
         # No dimensions in dataset type so we don't have to worry about
         # inserting dimension data or defining data IDs.
         self.datasetType = DatasetType(
-            "data", dimensions=(), storageClass="DataFrame", universe=self.butler.registry.dimensions
+            "data", dimensions=(), storageClass="DataFrame", universe=self.butler.dimensions
         )
         self.butler.registry.registerDatasetType(self.datasetType)
 
@@ -309,6 +340,19 @@ class ParquetFormatterDataFrameTestCase(unittest.TestCase):
         with self.assertRaises(ValueError):
             self.butler.get(self.datasetType, dataId={}, parameters={"columns": ["e"]})
 
+    def testSingleIndexDataFrameWithLists(self):
+        df1, allColumns = _makeSingleIndexDataFrame(include_lists=True)
+
+        self.butler.put(df1, self.datasetType, dataId={})
+        # Read the whole DataFrame.
+        df2 = self.butler.get(self.datasetType, dataId={})
+
+        # We need to check the list columns specially because they go
+        # from lists to arrays.
+        for col in ["l1", "l2", "l3"]:
+            for i in range(len(df1)):
+                self.assertTrue(np.all(df2[col].values[i] == df1[col].values[i]))
+
     def testMultiIndexDataFrame(self):
         df1 = _makeMultiIndexDataFrame()
 
@@ -383,12 +427,12 @@ class ParquetFormatterDataFrameTestCase(unittest.TestCase):
             "legacy_dataframe",
             dimensions=(),
             storageClass="DataFrame",
-            universe=self.butler.registry.dimensions,
+            universe=self.butler.dimensions,
         )
         self.butler.registry.registerDatasetType(legacy_type)
 
         data_id = {}
-        ref = DatasetRef(legacy_type, data_id, run="testLegacyDataFrame")
+        ref = DatasetRef(legacy_type, data_id, run=self.run)
         dataset = FileDataset(path=fname, refs=[ref], formatter=ParquetFormatter)
 
         self.butler.ingest(dataset, transfer="copy")
@@ -673,11 +717,12 @@ class ParquetFormatterArrowAstropyTestCase(unittest.TestCase):
         """Create a new butler root for each test."""
         self.root = makeTestTempDir(TESTDIR)
         config = Config(self.configFile)
-        self.butler = Butler(Butler.makeRepo(self.root, config=config), writeable=True, run="test_run")
+        self.run = "test_run"
+        self.butler = Butler(Butler.makeRepo(self.root, config=config), writeable=True, run=self.run)
         # No dimensions in dataset type so we don't have to worry about
         # inserting dimension data or defining data IDs.
         self.datasetType = DatasetType(
-            "data", dimensions=(), storageClass="ArrowAstropy", universe=self.butler.registry.dimensions
+            "data", dimensions=(), storageClass="ArrowAstropy", universe=self.butler.dimensions
         )
         self.butler.registry.registerDatasetType(self.datasetType)
 
@@ -785,12 +830,12 @@ class ParquetFormatterArrowAstropyTestCase(unittest.TestCase):
             "astropy_parquet",
             dimensions=(),
             storageClass="ArrowAstropy",
-            universe=self.butler.registry.dimensions,
+            universe=self.butler.dimensions,
         )
         self.butler.registry.registerDatasetType(astropy_type)
 
         data_id = {}
-        ref = DatasetRef(astropy_type, data_id, run="testAstropyParquet")
+        ref = DatasetRef(astropy_type, data_id, run=self.run)
         dataset = FileDataset(path=fname, refs=[ref], formatter=ParquetFormatter)
 
         self.butler.ingest(dataset, transfer="copy")
@@ -1004,7 +1049,7 @@ class ParquetFormatterArrowNumpyTestCase(unittest.TestCase):
         # No dimensions in dataset type so we don't have to worry about
         # inserting dimension data or defining data IDs.
         self.datasetType = DatasetType(
-            "data", dimensions=(), storageClass="ArrowNumpy", universe=self.butler.registry.dimensions
+            "data", dimensions=(), storageClass="ArrowNumpy", universe=self.butler.dimensions
         )
         self.butler.registry.registerDatasetType(self.datasetType)
 
@@ -1264,7 +1309,7 @@ class ParquetFormatterArrowTableTestCase(unittest.TestCase):
         # No dimensions in dataset type so we don't have to worry about
         # inserting dimension data or defining data IDs.
         self.datasetType = DatasetType(
-            "data", dimensions=(), storageClass="ArrowTable", universe=self.butler.registry.dimensions
+            "data", dimensions=(), storageClass="ArrowTable", universe=self.butler.dimensions
         )
         self.butler.registry.registerDatasetType(self.datasetType)
 
@@ -1571,7 +1616,7 @@ class ParquetFormatterArrowNumpyDictTestCase(unittest.TestCase):
         # No dimensions in dataset type so we don't have to worry about
         # inserting dimension data or defining data IDs.
         self.datasetType = DatasetType(
-            "data", dimensions=(), storageClass="ArrowNumpyDict", universe=self.butler.registry.dimensions
+            "data", dimensions=(), storageClass="ArrowNumpyDict", universe=self.butler.dimensions
         )
         self.butler.registry.registerDatasetType(self.datasetType)
 
@@ -1589,7 +1634,7 @@ class ParquetFormatterArrowNumpyDictTestCase(unittest.TestCase):
         # Read the columns.
         columns2 = self.butler.get(self.datasetType.componentTypeName("columns"), dataId={})
         self.assertEqual(len(columns2), len(dict1.keys()))
-        for i, name in enumerate(dict1.keys()):
+        for name in dict1:
             self.assertIn(name, columns2)
         # Read the rowcount.
         rowcount = self.butler.get(self.datasetType.componentTypeName("rowcount"), dataId={})
@@ -1693,7 +1738,7 @@ class ParquetFormatterArrowNumpyDictTestCase(unittest.TestCase):
         dict2 : `dict` [`str`, `np.ndarray`]
         """
         self.assertEqual(set(dict1.keys()), set(dict2.keys()))
-        for name in dict1.keys():
+        for name in dict1:
             self.assertEqual(dict1[name].dtype, dict2[name].dtype)
             self.assertTrue(np.all(dict1[name] == dict2[name]))
 
@@ -1754,6 +1799,14 @@ class ComputeRowGroupSizeTestCase(unittest.TestCase):
 
         self.assertGreater(row_group_size, 1_000_000)
 
+    @unittest.skipUnless(pd is not None, "Cannot run testRowGroupSizeDataFrameWithLists without pandas.")
+    def testRowGroupSizeDataFrameWithLists(self):
+        df = pd.DataFrame({"a": np.zeros(10), "b": [[0, 0]] * 10, "c": [[0.0, 0.0]] * 10, "d": [[]] * 10})
+        arrowTable = pandas_to_arrow(df)
+        row_group_size = compute_row_group_size(arrowTable.schema)
+
+        self.assertGreater(row_group_size, 1_000_000)
+
 
 if __name__ == "__main__":
     unittest.main()
