@@ -22,6 +22,7 @@
 import json
 import os
 import unittest
+import unittest.mock
 from typing import cast
 
 from lsst.daf.butler import (
@@ -33,11 +34,12 @@ from lsst.daf.butler import (
     Quantum,
     QuantumBackedButler,
     QuantumProvenanceData,
-    Registry,
     RegistryConfig,
     StorageClass,
 )
+from lsst.daf.butler.registry import _RegistryFactory
 from lsst.daf.butler.tests.utils import makeTestTempDir, removeTestTempDir
+from lsst.resources import ResourcePath
 
 TESTDIR = os.path.abspath(os.path.dirname(__file__))
 
@@ -53,7 +55,7 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
 
         # Make a butler and import dimension definitions.
         registryConfig = RegistryConfig(self.config.get("registry"))
-        Registry.createFromConfig(registryConfig, butlerRoot=self.root)
+        _RegistryFactory(registryConfig).create_from_config(butlerRoot=self.root)
         self.butler = Butler(self.config, writeable=True, run="RUN")
         self.butler.import_(filename=os.path.join(TESTDIR, "data", "registry", "base.yaml"))
 
@@ -101,15 +103,14 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
 
     def make_quantum(self, step: int = 1) -> Quantum:
         """Make a Quantum which includes datastore records."""
-
         if step == 1:
-            datastore_records = self.butler.datastore.export_records(self.all_input_refs)
+            datastore_records = self.butler._datastore.export_records(self.all_input_refs)
             predictedInputs = {self.datasetTypeInput: self.input_refs}
             outputs = {self.datasetTypeOutput: self.output_refs}
             initInputs = {self.datasetTypeInit: self.init_inputs_refs[0]}
         elif step == 2:
             # The result should be empty, this is just to test that it works.
-            datastore_records = self.butler.datastore.export_records(self.output_refs)
+            datastore_records = self.butler._datastore.export_records(self.output_refs)
             predictedInputs = {self.datasetTypeInput: self.output_refs}
             outputs = {self.datasetTypeOutput2: self.output_refs2}
             initInputs = {}
@@ -126,21 +127,39 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
 
     def test_initialize(self) -> None:
         """Test for initialize factory method"""
-
         quantum = self.make_quantum()
         qbb = QuantumBackedButler.initialize(
             config=self.config, quantum=quantum, dimensions=self.universe, dataset_types=self.dataset_types
         )
         self._test_factory(qbb)
 
+    def test_initialize_repo_index(self) -> None:
+        """Test for initialize using config file and repo index."""
+        # Store config to a file.
+        self.config.dumpToUri(self.root)
+
+        butler_index = Config()
+        butler_index["label"] = self.root
+        with ResourcePath.temporary_uri(suffix=".yaml") as index_path:
+            butler_index.dumpToUri(index_path)
+
+            with unittest.mock.patch.dict(os.environ, {"DAF_BUTLER_REPOSITORY_INDEX": str(index_path)}):
+                quantum = self.make_quantum()
+                qbb = QuantumBackedButler.initialize(
+                    config="label",
+                    quantum=quantum,
+                    dimensions=self.universe,
+                    dataset_types=self.dataset_types,
+                )
+                self._test_factory(qbb)
+
     def test_from_predicted(self) -> None:
         """Test for from_predicted factory method"""
-
-        datastore_records = self.butler.datastore.export_records(self.all_input_refs)
+        datastore_records = self.butler._datastore.export_records(self.all_input_refs)
         qbb = QuantumBackedButler.from_predicted(
             config=self.config,
-            predicted_inputs=[ref.getCheckedId() for ref in self.all_input_refs],
-            predicted_outputs=[ref.getCheckedId() for ref in self.output_refs],
+            predicted_inputs=[ref.id for ref in self.all_input_refs],
+            predicted_outputs=[ref.id for ref in self.output_refs],
             dimensions=self.universe,
             datastore_records=datastore_records,
             dataset_types=self.dataset_types,
@@ -149,10 +168,9 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
 
     def _test_factory(self, qbb: QuantumBackedButler) -> None:
         """Test state immediately after construction."""
-
         self.assertTrue(qbb.isWriteable())
-        self.assertEqual(qbb._predicted_inputs, set(ref.id for ref in self.all_input_refs))
-        self.assertEqual(qbb._predicted_outputs, set(ref.id for ref in self.output_refs))
+        self.assertEqual(qbb._predicted_inputs, {ref.id for ref in self.all_input_refs})
+        self.assertEqual(qbb._predicted_outputs, {ref.id for ref in self.output_refs})
         self.assertEqual(qbb._available_inputs, set())
         self.assertEqual(qbb._unavailable_inputs, set())
         self.assertEqual(qbb._actual_inputs, set())
@@ -160,7 +178,6 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
 
     def test_getput(self) -> None:
         """Test for getDirect/putDirect methods"""
-
         quantum = self.make_quantum()
         qbb = QuantumBackedButler.initialize(
             config=self.config, quantum=quantum, dimensions=self.universe, dataset_types=self.dataset_types
@@ -179,7 +196,7 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
 
         self.assertEqual(qbb._available_inputs, qbb._predicted_inputs)
         self.assertEqual(qbb._actual_inputs, qbb._predicted_inputs)
-        self.assertEqual(qbb._unavailable_inputs, set(ref.id for ref in self.missing_refs))
+        self.assertEqual(qbb._unavailable_inputs, {ref.id for ref in self.missing_refs})
 
         # Write all expected outputs.
         for ref in self.output_refs:
@@ -194,7 +211,6 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
 
     def test_getDeferred(self) -> None:
         """Test for getDirectDeferred method"""
-
         quantum = self.make_quantum()
         qbb = QuantumBackedButler.initialize(
             config=self.config, quantum=quantum, dimensions=self.universe, dataset_types=self.dataset_types
@@ -214,13 +230,12 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
                 data.get()
 
         # _avalable_inputs is not
-        self.assertEqual(qbb._available_inputs, set(ref.id for ref in input_refs + self.init_inputs_refs))
-        self.assertEqual(qbb._actual_inputs, set(ref.id for ref in input_refs + self.init_inputs_refs))
-        self.assertEqual(qbb._unavailable_inputs, set(ref.id for ref in self.missing_refs))
+        self.assertEqual(qbb._available_inputs, {ref.id for ref in input_refs + self.init_inputs_refs})
+        self.assertEqual(qbb._actual_inputs, {ref.id for ref in input_refs + self.init_inputs_refs})
+        self.assertEqual(qbb._unavailable_inputs, {ref.id for ref in self.missing_refs})
 
     def test_datasetExistsDirect(self) -> None:
-        """Test for datasetExistsDirect method"""
-
+        """Test for dataset existence method"""
         quantum = self.make_quantum()
         qbb = QuantumBackedButler.initialize(
             config=self.config, quantum=quantum, dimensions=self.universe, dataset_types=self.dataset_types
@@ -229,23 +244,32 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
         # get some input data
         input_refs = self.input_refs[:2]
         for ref in input_refs:
-            exists = qbb.datasetExistsDirect(ref)
+            exists = qbb.stored(ref)
             self.assertTrue(exists)
         for ref in self.init_inputs_refs:
-            exists = qbb.datasetExistsDirect(ref)
+            exists = qbb.stored(ref)
             self.assertTrue(exists)
         for ref in self.missing_refs:
-            exists = qbb.datasetExistsDirect(ref)
+            exists = qbb.stored(ref)
             self.assertFalse(exists)
 
+        # Now do the same checks in bulk.
+        missing_set = set(self.missing_refs)
+        refs = input_refs + self.init_inputs_refs + self.missing_refs
+        stored_many = qbb.stored_many(refs)
+        for ref, stored in stored_many.items():
+            if ref in missing_set:
+                self.assertFalse(stored)
+            else:
+                self.assertTrue(stored)
+
         # _available_inputs is not
-        self.assertEqual(qbb._available_inputs, set(ref.id for ref in input_refs + self.init_inputs_refs))
+        self.assertEqual(qbb._available_inputs, {ref.id for ref in input_refs + self.init_inputs_refs})
         self.assertEqual(qbb._actual_inputs, set())
         self.assertEqual(qbb._unavailable_inputs, set())  # this is not consistent with getDirect?
 
     def test_markInputUnused(self) -> None:
         """Test for markInputUnused method"""
-
         quantum = self.make_quantum()
         qbb = QuantumBackedButler.initialize(
             config=self.config, quantum=quantum, dimensions=self.universe, dataset_types=self.dataset_types
@@ -263,13 +287,10 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
         self.assertEqual(qbb._actual_inputs, qbb._predicted_inputs)
 
         qbb.markInputUnused(self.input_refs[0])
-        self.assertEqual(
-            qbb._actual_inputs, set(ref.id for ref in self.input_refs[1:] + self.init_inputs_refs)
-        )
+        self.assertEqual(qbb._actual_inputs, {ref.id for ref in self.input_refs[1:] + self.init_inputs_refs})
 
     def test_pruneDatasets(self) -> None:
         """Test for pruneDatasets methods"""
-
         quantum = self.make_quantum()
         qbb = QuantumBackedButler.initialize(
             config=self.config, quantum=quantum, dimensions=self.universe, dataset_types=self.dataset_types
@@ -295,26 +316,25 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
         # Disassociate only.
         ref = self.output_refs[0]
         qbb.pruneDatasets([ref], disassociate=False, unstore=True, purge=False)
-        self.assertFalse(qbb.datasetExistsDirect(ref))
+        self.assertFalse(qbb.stored(ref))
         with self.assertRaises(FileNotFoundError):
             data = qbb.get(ref)
 
         # can store it again
         qbb.put({"data": cast(int, ref.dataId["detector"]) ** 2}, ref)
-        self.assertTrue(qbb.datasetExistsDirect(ref))
+        self.assertTrue(qbb.stored(ref))
 
         # Purge completely.
         ref = self.output_refs[1]
         qbb.pruneDatasets([ref], disassociate=True, unstore=True, purge=True)
-        self.assertFalse(qbb.datasetExistsDirect(ref))
+        self.assertFalse(qbb.stored(ref))
         with self.assertRaises(FileNotFoundError):
             data = qbb.get(ref)
         qbb.put({"data": cast(int, ref.dataId["detector"]) ** 2}, ref)
-        self.assertTrue(qbb.datasetExistsDirect(ref))
+        self.assertTrue(qbb.stored(ref))
 
     def test_extract_provenance_data(self) -> None:
         """Test for extract_provenance_data method"""
-
         quantum = self.make_quantum()
         qbb = QuantumBackedButler.initialize(
             config=self.config, quantum=quantum, dimensions=self.universe, dataset_types=self.dataset_types
@@ -332,11 +352,11 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
         prov_json = provenance1.json()
         provenance2 = QuantumProvenanceData.direct(**json.loads(prov_json))
         for provenance in (provenance1, provenance2):
-            input_ids = set(ref.id for ref in self.input_refs + self.init_inputs_refs)
+            input_ids = {ref.id for ref in self.input_refs + self.init_inputs_refs}
             self.assertEqual(provenance.predicted_inputs, input_ids)
             self.assertEqual(provenance.available_inputs, input_ids)
             self.assertEqual(provenance.actual_inputs, input_ids)
-            output_ids = set(ref.id for ref in self.output_refs)
+            output_ids = {ref.id for ref in self.output_refs}
             self.assertEqual(provenance.predicted_outputs, output_ids)
             self.assertEqual(provenance.actual_outputs, output_ids)
             datastore_name = "FileDatastore@<butlerRoot>/datastore"
@@ -348,13 +368,12 @@ class QuantumBackedButlerTestCase(unittest.TestCase):
             self.assertEqual(set(datastore_records.records.keys()), {class_name})
             self.assertEqual(set(datastore_records.records[class_name].keys()), {table_name})
             self.assertEqual(
-                set(record["dataset_id"] for record in datastore_records.records[class_name][table_name]),
+                {record["dataset_id"] for record in datastore_records.records[class_name][table_name]},
                 output_ids,
             )
 
     def test_collect_and_transfer(self) -> None:
         """Test for collect_and_transfer method"""
-
         quantum1 = self.make_quantum(1)
         qbb1 = QuantumBackedButler.initialize(
             config=self.config, quantum=quantum1, dimensions=self.universe, dataset_types=self.dataset_types
