@@ -26,7 +26,6 @@ import os
 import re
 import tempfile
 import unittest
-import uuid
 from typing import Any
 
 try:
@@ -35,8 +34,8 @@ except ImportError:
     np = None
 
 import astropy.time
-from lsst.daf.butler import Butler, ButlerConfig, CollectionType, DatasetRef, DatasetType, Registry, Timespan
-from lsst.daf.butler.registry import ConflictingDefinitionError, RegistryConfig, RegistryDefaults
+from lsst.daf.butler import Butler, ButlerConfig, CollectionType, DatasetId, DatasetRef, DatasetType, Timespan
+from lsst.daf.butler.registry import RegistryConfig, RegistryDefaults, _RegistryFactory
 from lsst.daf.butler.tests import DatastoreMock
 from lsst.daf.butler.tests.utils import makeTestTempDir, removeTestTempDir
 
@@ -53,7 +52,6 @@ class SimpleButlerTestCase(unittest.TestCase):
         "lsst.daf.butler.registry.datasets.byDimensions.ByDimensionsDatasetRecordStorageManagerUUID"
     )
     datasetsImportFile = "datasets-uuid.yaml"
-    datasetsIdType = uuid.UUID
 
     def setUp(self):
         self.root = makeTestTempDir(TESTDIR)
@@ -73,11 +71,7 @@ class SimpleButlerTestCase(unittest.TestCase):
 
         # have to make a registry first
         registryConfig = RegistryConfig(config.get("registry"))
-        if self.datasetsIdType is int:
-            with self.assertWarns(FutureWarning):
-                Registry.createFromConfig(registryConfig)
-        else:
-            Registry.createFromConfig(registryConfig)
+        _RegistryFactory(registryConfig).create_from_config()
 
         butler = Butler(config, **kwargs)
         DatastoreMock.apply(butler)
@@ -91,7 +85,7 @@ class SimpleButlerTestCase(unittest.TestCase):
         method returns unresolved DatasetRef, for round-trip-safe repos it
         returns unchanged ref.
         """
-        return ref if self.datasetsIdType is uuid.UUID else ref.unresolved()
+        return ref
 
     def testReadBackwardsCompatibility(self):
         """Test that we can read an export file written by a previous version
@@ -109,7 +103,7 @@ class SimpleButlerTestCase(unittest.TestCase):
         # Spot-check a few things, but the most important test is just that
         # the above does not raise.
         self.assertGreaterEqual(
-            set(record.id for record in butler.registry.queryDimensionRecords("detector", instrument="HSC")),
+            {record.id for record in butler.registry.queryDimensionRecords("detector", instrument="HSC")},
             set(range(104)),  # should have all science CCDs; may have some focus ones.
         )
         self.assertGreaterEqual(
@@ -158,8 +152,8 @@ class SimpleButlerTestCase(unittest.TestCase):
             butler2.import_(filename=file.name)
         datasets1 = list(butler1.registry.queryDatasets(..., collections=...))
         datasets2 = list(butler2.registry.queryDatasets(..., collections=...))
-        self.assertTrue(all(isinstance(ref.id, self.datasetsIdType) for ref in datasets1))
-        self.assertTrue(all(isinstance(ref.id, self.datasetsIdType) for ref in datasets2))
+        self.assertTrue(all(isinstance(ref.id, DatasetId) for ref in datasets1))
+        self.assertTrue(all(isinstance(ref.id, DatasetId) for ref in datasets2))
         self.assertCountEqual(
             [self.comparableRef(ref) for ref in datasets1],
             [self.comparableRef(ref) for ref in datasets2],
@@ -169,8 +163,6 @@ class SimpleButlerTestCase(unittest.TestCase):
         """Test exporting dimension records and datasets from a repo and then
         importing them all back in again twice.
         """
-        if self.datasetsIdType is not uuid.UUID:
-            self.skipTest("This test can only work for UUIDs")
         # Import data to play with.
         butler1 = self.makeButler(writeable=True)
         butler1.import_(filename=os.path.join(TESTDIR, "data", "registry", "base.yaml"))
@@ -186,39 +178,13 @@ class SimpleButlerTestCase(unittest.TestCase):
             butler2.import_(filename=file.name)
         datasets1 = list(butler1.registry.queryDatasets(..., collections=...))
         datasets2 = list(butler2.registry.queryDatasets(..., collections=...))
-        self.assertTrue(all(isinstance(ref.id, self.datasetsIdType) for ref in datasets1))
-        self.assertTrue(all(isinstance(ref.id, self.datasetsIdType) for ref in datasets2))
+        self.assertTrue(all(isinstance(ref.id, DatasetId) for ref in datasets1))
+        self.assertTrue(all(isinstance(ref.id, DatasetId) for ref in datasets2))
         self.assertCountEqual(
             [self.comparableRef(ref) for ref in datasets1],
             [self.comparableRef(ref) for ref in datasets2],
         )
 
-    def testDatasetImportReuseIds(self):
-        """Test for import that should preserve dataset IDs.
-
-        This test assumes that dataset IDs in datasets YAML are different from
-        what auto-incremental insert would produce.
-        """
-        if self.datasetsIdType is not int:
-            self.skipTest("This test can only work for UUIDs")
-        # Import data to play with.
-        butler = self.makeButler(writeable=True)
-        butler.import_(filename=os.path.join(TESTDIR, "data", "registry", "base.yaml"))
-        filename = os.path.join(TESTDIR, "data", "registry", self.datasetsImportFile)
-        butler.import_(filename=filename, reuseIds=True)
-        datasets = list(butler.registry.queryDatasets(..., collections=...))
-        self.assertTrue(all(isinstance(ref.id, self.datasetsIdType) for ref in datasets))
-        # IDs are copied from YAML, list needs to be updated if file contents
-        # is changed.
-        self.assertCountEqual(
-            [ref.id for ref in datasets],
-            [1001, 1002, 1003, 1010, 1020, 1030, 2001, 2002, 2003, 2010, 2020, 2030, 2040],
-        )
-
-        # Try once again, it will raise
-        with self.assertRaises(ConflictingDefinitionError):
-            butler.import_(filename=filename, reuseIds=True)
-
     def testCollectionTransfers(self):
         """Test exporting and then importing collections of various types."""
         # Populate a registry with some datasets.
@@ -298,7 +264,6 @@ class SimpleButlerTestCase(unittest.TestCase):
 
     def testButlerGet(self):
         """Test that butler.get can work with different variants."""
-
         # Import data to play with.
         butler = self.makeButler(writeable=True)
         butler.import_(filename=os.path.join(TESTDIR, "data", "registry", "base.yaml"))
@@ -312,7 +277,6 @@ class SimpleButlerTestCase(unittest.TestCase):
 
         # Create a numpy integer to check that works fine
         detector_np = np.int64(2) if np else 2
-        print(type(detector_np))
 
         # Try to get it using different variations of dataId + keyword
         # arguments
@@ -571,7 +535,7 @@ class SimpleButlerTestCase(unittest.TestCase):
         butler.registry.insertDimensionData("instrument", {"name": "Cam2"})
         camera = DatasetType(
             "camera",
-            dimensions=butler.registry.dimensions["instrument"].graph,
+            dimensions=butler.dimensions["instrument"].graph,
             storageClass="Camera",
         )
         butler.registry.registerDatasetType(camera)
@@ -590,6 +554,17 @@ class SimpleButlerTestCase(unittest.TestCase):
         self.assertIsNone(butler3.registry.defaults.run, None)
         self.assertEqual(butler3.registry.defaults.dataId.byName(), {"instrument": "Cam2"})
 
+        # Check that repr() does not fail.
+        defaults = RegistryDefaults(collections=["imported_g"], run="test")
+        r = repr(defaults)
+        self.assertIn("collections=('imported_g',)", r)
+        self.assertIn("run='test'", r)
+
+        defaults = RegistryDefaults(run="test", instrument="DummyCam", skypix="pix")
+        r = repr(defaults)
+        self.assertIn("skypix='pix'", r)
+        self.assertIn("instrument='DummyCam'", r)
+
     def testJson(self):
         """Test JSON serialization mediated by registry."""
         butler = self.makeButler(writeable=True)
@@ -618,7 +593,7 @@ class SimpleButlerTestCase(unittest.TestCase):
 
                 # for minimal=False case also do a test without registry
                 if not minimal:
-                    from_json = type(test_item).from_json(json_str, universe=butler.registry.dimensions)
+                    from_json = type(test_item).from_json(json_str, universe=butler.dimensions)
                     self.assertEqual(from_json, test_item, msg=f"From JSON '{json_str}' using universe")
 
     def testJsonDimensionRecordsAndHtmlRepresentation(self):
@@ -647,7 +622,6 @@ class SimpleButlerTestCase(unittest.TestCase):
 
     def testWildcardQueries(self):
         """Test that different collection type queries work."""
-
         # Import data to play with.
         butler = self.makeButler(writeable=True)
         butler.import_(filename=os.path.join(TESTDIR, "data", "registry", "base.yaml"))
@@ -682,11 +656,7 @@ class SimpleButlerMixedUUIDTestCase(SimpleButlerTestCase):
     loads datasets from YAML file with integer IDs.
     """
 
-    datasetsManager = (
-        "lsst.daf.butler.registry.datasets.byDimensions.ByDimensionsDatasetRecordStorageManagerUUID"
-    )
     datasetsImportFile = "datasets.yaml"
-    datasetsIdType = uuid.UUID
 
 
 if __name__ == "__main__":
